sqlglot.dialects.presto API documentation

Edit on GitHub

# sqlglot.dialects.presto

View Source

      1from __future__ import annotations
      2
      3import typing as t
      4
      5from sqlglot import exp, generator, parser, tokens, transforms
      6from sqlglot.dialects.dialect import (
      7    Dialect,
      8    NormalizationStrategy,
      9    binary_from_function,
     10    bool_xor_sql,
     11    date_trunc_to_time,
     12    datestrtodate_sql,
     13    encode_decode_sql,
     14    build_formatted_time,
     15    if_sql,
     16    left_to_substring_sql,
     17    no_ilike_sql,
     18    no_pivot_sql,
     19    no_safe_divide_sql,
     20    no_timestamp_sql,
     21    regexp_extract_sql,
     22    rename_func,
     23    right_to_substring_sql,
     24    struct_extract_sql,
     25    str_position_sql,
     26    timestamptrunc_sql,
     27    timestrtotime_sql,
     28    ts_or_ds_add_cast,
     29    unit_to_str,
     30)
     31from sqlglot.dialects.hive import Hive
     32from sqlglot.dialects.mysql import MySQL
     33from sqlglot.helper import apply_index_offset, seq_get
     34from sqlglot.tokens import TokenType
     35
     36
     37def _explode_to_unnest_sql(self: Presto.Generator, expression: exp.Lateral) -> str:
     38    if isinstance(expression.this, exp.Explode):
     39        return self.sql(
     40            exp.Join(
     41                this=exp.Unnest(
     42                    expressions=[expression.this.this],
     43                    alias=expression.args.get("alias"),
     44                    offset=isinstance(expression.this, exp.Posexplode),
     45                ),
     46                kind="cross",
     47            )
     48        )
     49    return self.lateral_sql(expression)
     50
     51
     52def _initcap_sql(self: Presto.Generator, expression: exp.Initcap) -> str:
     53    regex = r"(\w)(\w*)"
     54    return f"REGEXP_REPLACE({self.sql(expression, 'this')}, '{regex}', x -> UPPER(x[1]) || LOWER(x[2]))"
     55
     56
     57def _no_sort_array(self: Presto.Generator, expression: exp.SortArray) -> str:
     58    if expression.args.get("asc") == exp.false():
     59        comparator = "(a, b) -> CASE WHEN a < b THEN 1 WHEN a > b THEN -1 ELSE 0 END"
     60    else:
     61        comparator = None
     62    return self.func("ARRAY_SORT", expression.this, comparator)
     63
     64
     65def _schema_sql(self: Presto.Generator, expression: exp.Schema) -> str:
     66    if isinstance(expression.parent, exp.Property):
     67        columns = ", ".join(f"'{c.name}'" for c in expression.expressions)
     68        return f"ARRAY[{columns}]"
     69
     70    if expression.parent:
     71        for schema in expression.parent.find_all(exp.Schema):
     72            column_defs = schema.find_all(exp.ColumnDef)
     73            if column_defs and isinstance(schema.parent, exp.Property):
     74                expression.expressions.extend(column_defs)
     75
     76    return self.schema_sql(expression)
     77
     78
     79def _quantile_sql(self: Presto.Generator, expression: exp.Quantile) -> str:
     80    self.unsupported("Presto does not support exact quantiles")
     81    return self.func("APPROX_PERCENTILE", expression.this, expression.args.get("quantile"))
     82
     83
     84def _str_to_time_sql(
     85    self: Presto.Generator, expression: exp.StrToDate | exp.StrToTime | exp.TsOrDsToDate
     86) -> str:
     87    return self.func("DATE_PARSE", expression.this, self.format_time(expression))
     88
     89
     90def _ts_or_ds_to_date_sql(self: Presto.Generator, expression: exp.TsOrDsToDate) -> str:
     91    time_format = self.format_time(expression)
     92    if time_format and time_format not in (Presto.TIME_FORMAT, Presto.DATE_FORMAT):
     93        return self.sql(exp.cast(_str_to_time_sql(self, expression), exp.DataType.Type.DATE))
     94    return self.sql(
     95        exp.cast(exp.cast(expression.this, exp.DataType.Type.TIMESTAMP), exp.DataType.Type.DATE)
     96    )
     97
     98
     99def _ts_or_ds_add_sql(self: Presto.Generator, expression: exp.TsOrDsAdd) -> str:
    100    expression = ts_or_ds_add_cast(expression)
    101    unit = unit_to_str(expression)
    102    return self.func("DATE_ADD", unit, expression.expression, expression.this)
    103
    104
    105def _ts_or_ds_diff_sql(self: Presto.Generator, expression: exp.TsOrDsDiff) -> str:
    106    this = exp.cast(expression.this, exp.DataType.Type.TIMESTAMP)
    107    expr = exp.cast(expression.expression, exp.DataType.Type.TIMESTAMP)
    108    unit = unit_to_str(expression)
    109    return self.func("DATE_DIFF", unit, expr, this)
    110
    111
    112def _build_approx_percentile(args: t.List) -> exp.Expression:
    113    if len(args) == 4:
    114        return exp.ApproxQuantile(
    115            this=seq_get(args, 0),
    116            weight=seq_get(args, 1),
    117            quantile=seq_get(args, 2),
    118            accuracy=seq_get(args, 3),
    119        )
    120    if len(args) == 3:
    121        return exp.ApproxQuantile(
    122            this=seq_get(args, 0), quantile=seq_get(args, 1), accuracy=seq_get(args, 2)
    123        )
    124    return exp.ApproxQuantile.from_arg_list(args)
    125
    126
    127def _build_from_unixtime(args: t.List) -> exp.Expression:
    128    if len(args) == 3:
    129        return exp.UnixToTime(
    130            this=seq_get(args, 0),
    131            hours=seq_get(args, 1),
    132            minutes=seq_get(args, 2),
    133        )
    134    if len(args) == 2:
    135        return exp.UnixToTime(this=seq_get(args, 0), zone=seq_get(args, 1))
    136
    137    return exp.UnixToTime.from_arg_list(args)
    138
    139
    140def _unnest_sequence(expression: exp.Expression) -> exp.Expression:
    141    if isinstance(expression, exp.Table):
    142        if isinstance(expression.this, exp.GenerateSeries):
    143            unnest = exp.Unnest(expressions=[expression.this])
    144
    145            if expression.alias:
    146                return exp.alias_(unnest, alias="_u", table=[expression.alias], copy=False)
    147            return unnest
    148    return expression
    149
    150
    151def _first_last_sql(self: Presto.Generator, expression: exp.Func) -> str:
    152    """
    153    Trino doesn't support FIRST / LAST as functions, but they're valid in the context
    154    of MATCH_RECOGNIZE, so we need to preserve them in that case. In all other cases
    155    they're converted into an ARBITRARY call.
    156
    157    Reference: https://trino.io/docs/current/sql/match-recognize.html#logical-navigation-functions
    158    """
    159    if isinstance(expression.find_ancestor(exp.MatchRecognize, exp.Select), exp.MatchRecognize):
    160        return self.function_fallback_sql(expression)
    161
    162    return rename_func("ARBITRARY")(self, expression)
    163
    164
    165def _unix_to_time_sql(self: Presto.Generator, expression: exp.UnixToTime) -> str:
    166    scale = expression.args.get("scale")
    167    timestamp = self.sql(expression, "this")
    168    if scale in (None, exp.UnixToTime.SECONDS):
    169        return rename_func("FROM_UNIXTIME")(self, expression)
    170
    171    return f"FROM_UNIXTIME(CAST({timestamp} AS DOUBLE) / POW(10, {scale}))"
    172
    173
    174def _to_int(expression: exp.Expression) -> exp.Expression:
    175    if not expression.type:
    176        from sqlglot.optimizer.annotate_types import annotate_types
    177
    178        annotate_types(expression)
    179    if expression.type and expression.type.this not in exp.DataType.INTEGER_TYPES:
    180        return exp.cast(expression, to=exp.DataType.Type.BIGINT)
    181    return expression
    182
    183
    184def _build_to_char(args: t.List) -> exp.TimeToStr:
    185    fmt = seq_get(args, 1)
    186    if isinstance(fmt, exp.Literal):
    187        # We uppercase this to match Teradata's format mapping keys
    188        fmt.set("this", fmt.this.upper())
    189
    190    # We use "teradata" on purpose here, because the time formats are different in Presto.
    191    # See https://prestodb.io/docs/current/functions/teradata.html?highlight=to_char#to_char
    192    return build_formatted_time(exp.TimeToStr, "teradata")(args)
    193
    194
    195class Presto(Dialect):
    196    INDEX_OFFSET = 1
    197    NULL_ORDERING = "nulls_are_last"
    198    TIME_FORMAT = MySQL.TIME_FORMAT
    199    TIME_MAPPING = MySQL.TIME_MAPPING
    200    STRICT_STRING_CONCAT = True
    201    SUPPORTS_SEMI_ANTI_JOIN = False
    202    TYPED_DIVISION = True
    203    TABLESAMPLE_SIZE_IS_PERCENT = True
    204    LOG_BASE_FIRST: t.Optional[bool] = None
    205
    206    # https://github.com/trinodb/trino/issues/17
    207    # https://github.com/trinodb/trino/issues/12289
    208    # https://github.com/prestodb/presto/issues/2863
    209    NORMALIZATION_STRATEGY = NormalizationStrategy.CASE_INSENSITIVE
    210
    211    class Tokenizer(tokens.Tokenizer):
    212        UNICODE_STRINGS = [
    213            (prefix + q, q)
    214            for q in t.cast(t.List[str], tokens.Tokenizer.QUOTES)
    215            for prefix in ("U&", "u&")
    216        ]
    217
    218        KEYWORDS = {
    219            **tokens.Tokenizer.KEYWORDS,
    220            "START": TokenType.BEGIN,
    221            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    222            "ROW": TokenType.STRUCT,
    223            "IPADDRESS": TokenType.IPADDRESS,
    224            "IPPREFIX": TokenType.IPPREFIX,
    225        }
    226
    227        KEYWORDS.pop("QUALIFY")
    228
    229    class Parser(parser.Parser):
    230        VALUES_FOLLOWED_BY_PAREN = False
    231
    232        FUNCTIONS = {
    233            **parser.Parser.FUNCTIONS,
    234            "ARBITRARY": exp.AnyValue.from_arg_list,
    235            "APPROX_DISTINCT": exp.ApproxDistinct.from_arg_list,
    236            "APPROX_PERCENTILE": _build_approx_percentile,
    237            "BITWISE_AND": binary_from_function(exp.BitwiseAnd),
    238            "BITWISE_NOT": lambda args: exp.BitwiseNot(this=seq_get(args, 0)),
    239            "BITWISE_OR": binary_from_function(exp.BitwiseOr),
    240            "BITWISE_XOR": binary_from_function(exp.BitwiseXor),
    241            "CARDINALITY": exp.ArraySize.from_arg_list,
    242            "CONTAINS": exp.ArrayContains.from_arg_list,
    243            "DATE_ADD": lambda args: exp.DateAdd(
    244                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    245            ),
    246            "DATE_DIFF": lambda args: exp.DateDiff(
    247                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    248            ),
    249            "DATE_FORMAT": build_formatted_time(exp.TimeToStr, "presto"),
    250            "DATE_PARSE": build_formatted_time(exp.StrToTime, "presto"),
    251            "DATE_TRUNC": date_trunc_to_time,
    252            "ELEMENT_AT": lambda args: exp.Bracket(
    253                this=seq_get(args, 0), expressions=[seq_get(args, 1)], offset=1, safe=True
    254            ),
    255            "FROM_HEX": exp.Unhex.from_arg_list,
    256            "FROM_UNIXTIME": _build_from_unixtime,
    257            "FROM_UTF8": lambda args: exp.Decode(
    258                this=seq_get(args, 0), replace=seq_get(args, 1), charset=exp.Literal.string("utf-8")
    259            ),
    260            "NOW": exp.CurrentTimestamp.from_arg_list,
    261            "REGEXP_EXTRACT": lambda args: exp.RegexpExtract(
    262                this=seq_get(args, 0), expression=seq_get(args, 1), group=seq_get(args, 2)
    263            ),
    264            "REGEXP_REPLACE": lambda args: exp.RegexpReplace(
    265                this=seq_get(args, 0),
    266                expression=seq_get(args, 1),
    267                replacement=seq_get(args, 2) or exp.Literal.string(""),
    268            ),
    269            "ROW": exp.Struct.from_arg_list,
    270            "SEQUENCE": exp.GenerateSeries.from_arg_list,
    271            "SET_AGG": exp.ArrayUniqueAgg.from_arg_list,
    272            "SPLIT_TO_MAP": exp.StrToMap.from_arg_list,
    273            "STRPOS": lambda args: exp.StrPosition(
    274                this=seq_get(args, 0), substr=seq_get(args, 1), instance=seq_get(args, 2)
    275            ),
    276            "TO_CHAR": _build_to_char,
    277            "TO_HEX": exp.Hex.from_arg_list,
    278            "TO_UNIXTIME": exp.TimeToUnix.from_arg_list,
    279            "TO_UTF8": lambda args: exp.Encode(
    280                this=seq_get(args, 0), charset=exp.Literal.string("utf-8")
    281            ),
    282        }
    283
    284        FUNCTION_PARSERS = parser.Parser.FUNCTION_PARSERS.copy()
    285        FUNCTION_PARSERS.pop("TRIM")
    286
    287    class Generator(generator.Generator):
    288        INTERVAL_ALLOWS_PLURAL_FORM = False
    289        JOIN_HINTS = False
    290        TABLE_HINTS = False
    291        QUERY_HINTS = False
    292        IS_BOOL_ALLOWED = False
    293        TZ_TO_WITH_TIME_ZONE = True
    294        NVL2_SUPPORTED = False
    295        STRUCT_DELIMITER = ("(", ")")
    296        LIMIT_ONLY_LITERALS = True
    297        SUPPORTS_SINGLE_ARG_CONCAT = False
    298        LIKE_PROPERTY_INSIDE_SCHEMA = True
    299        MULTI_ARG_DISTINCT = False
    300        SUPPORTS_TO_NUMBER = False
    301
    302        PROPERTIES_LOCATION = {
    303            **generator.Generator.PROPERTIES_LOCATION,
    304            exp.LocationProperty: exp.Properties.Location.UNSUPPORTED,
    305            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    306        }
    307
    308        TYPE_MAPPING = {
    309            **generator.Generator.TYPE_MAPPING,
    310            exp.DataType.Type.INT: "INTEGER",
    311            exp.DataType.Type.FLOAT: "REAL",
    312            exp.DataType.Type.BINARY: "VARBINARY",
    313            exp.DataType.Type.TEXT: "VARCHAR",
    314            exp.DataType.Type.TIMETZ: "TIME",
    315            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    316            exp.DataType.Type.STRUCT: "ROW",
    317            exp.DataType.Type.DATETIME: "TIMESTAMP",
    318            exp.DataType.Type.DATETIME64: "TIMESTAMP",
    319        }
    320
    321        TRANSFORMS = {
    322            **generator.Generator.TRANSFORMS,
    323            exp.AnyValue: rename_func("ARBITRARY"),
    324            exp.ApproxDistinct: lambda self, e: self.func(
    325                "APPROX_DISTINCT", e.this, e.args.get("accuracy")
    326            ),
    327            exp.ApproxQuantile: rename_func("APPROX_PERCENTILE"),
    328            exp.ArgMax: rename_func("MAX_BY"),
    329            exp.ArgMin: rename_func("MIN_BY"),
    330            exp.Array: lambda self, e: f"ARRAY[{self.expressions(e, flat=True)}]",
    331            exp.ArrayAny: rename_func("ANY_MATCH"),
    332            exp.ArrayConcat: rename_func("CONCAT"),
    333            exp.ArrayContains: rename_func("CONTAINS"),
    334            exp.ArraySize: rename_func("CARDINALITY"),
    335            exp.ArrayToString: rename_func("ARRAY_JOIN"),
    336            exp.ArrayUniqueAgg: rename_func("SET_AGG"),
    337            exp.AtTimeZone: rename_func("AT_TIMEZONE"),
    338            exp.BitwiseAnd: lambda self, e: self.func("BITWISE_AND", e.this, e.expression),
    339            exp.BitwiseLeftShift: lambda self, e: self.func(
    340                "BITWISE_ARITHMETIC_SHIFT_LEFT", e.this, e.expression
    341            ),
    342            exp.BitwiseNot: lambda self, e: self.func("BITWISE_NOT", e.this),
    343            exp.BitwiseOr: lambda self, e: self.func("BITWISE_OR", e.this, e.expression),
    344            exp.BitwiseRightShift: lambda self, e: self.func(
    345                "BITWISE_ARITHMETIC_SHIFT_RIGHT", e.this, e.expression
    346            ),
    347            exp.BitwiseXor: lambda self, e: self.func("BITWISE_XOR", e.this, e.expression),
    348            exp.Cast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    349            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    350            exp.DateAdd: lambda self, e: self.func(
    351                "DATE_ADD",
    352                unit_to_str(e),
    353                _to_int(e.expression),
    354                e.this,
    355            ),
    356            exp.DateDiff: lambda self, e: self.func(
    357                "DATE_DIFF", unit_to_str(e), e.expression, e.this
    358            ),
    359            exp.DateStrToDate: datestrtodate_sql,
    360            exp.DateToDi: lambda self,
    361            e: f"CAST(DATE_FORMAT({self.sql(e, 'this')}, {Presto.DATEINT_FORMAT}) AS INT)",
    362            exp.DateSub: lambda self, e: self.func(
    363                "DATE_ADD",
    364                unit_to_str(e),
    365                _to_int(e.expression * -1),
    366                e.this,
    367            ),
    368            exp.Decode: lambda self, e: encode_decode_sql(self, e, "FROM_UTF8"),
    369            exp.DiToDate: lambda self,
    370            e: f"CAST(DATE_PARSE(CAST({self.sql(e, 'this')} AS VARCHAR), {Presto.DATEINT_FORMAT}) AS DATE)",
    371            exp.Encode: lambda self, e: encode_decode_sql(self, e, "TO_UTF8"),
    372            exp.FileFormatProperty: lambda self, e: f"FORMAT='{e.name.upper()}'",
    373            exp.First: _first_last_sql,
    374            exp.FirstValue: _first_last_sql,
    375            exp.FromTimeZone: lambda self,
    376            e: f"WITH_TIMEZONE({self.sql(e, 'this')}, {self.sql(e, 'zone')}) AT TIME ZONE 'UTC'",
    377            exp.Group: transforms.preprocess([transforms.unalias_group]),
    378            exp.GroupConcat: lambda self, e: self.func(
    379                "ARRAY_JOIN", self.func("ARRAY_AGG", e.this), e.args.get("separator")
    380            ),
    381            exp.Hex: rename_func("TO_HEX"),
    382            exp.If: if_sql(),
    383            exp.ILike: no_ilike_sql,
    384            exp.Initcap: _initcap_sql,
    385            exp.ParseJSON: rename_func("JSON_PARSE"),
    386            exp.Last: _first_last_sql,
    387            exp.LastValue: _first_last_sql,
    388            exp.LastDay: lambda self, e: self.func("LAST_DAY_OF_MONTH", e.this),
    389            exp.Lateral: _explode_to_unnest_sql,
    390            exp.Left: left_to_substring_sql,
    391            exp.Levenshtein: rename_func("LEVENSHTEIN_DISTANCE"),
    392            exp.LogicalAnd: rename_func("BOOL_AND"),
    393            exp.LogicalOr: rename_func("BOOL_OR"),
    394            exp.Pivot: no_pivot_sql,
    395            exp.Quantile: _quantile_sql,
    396            exp.RegexpExtract: regexp_extract_sql,
    397            exp.Right: right_to_substring_sql,
    398            exp.SafeDivide: no_safe_divide_sql,
    399            exp.Schema: _schema_sql,
    400            exp.SchemaCommentProperty: lambda self, e: self.naked_property(e),
    401            exp.Select: transforms.preprocess(
    402                [
    403                    transforms.eliminate_qualify,
    404                    transforms.eliminate_distinct_on,
    405                    transforms.explode_to_unnest(1),
    406                    transforms.eliminate_semi_and_anti_joins,
    407                ]
    408            ),
    409            exp.SortArray: _no_sort_array,
    410            exp.StrPosition: lambda self, e: str_position_sql(self, e, generate_instance=True),
    411            exp.StrToDate: lambda self, e: f"CAST({_str_to_time_sql(self, e)} AS DATE)",
    412            exp.StrToMap: rename_func("SPLIT_TO_MAP"),
    413            exp.StrToTime: _str_to_time_sql,
    414            exp.StructExtract: struct_extract_sql,
    415            exp.Table: transforms.preprocess([_unnest_sequence]),
    416            exp.Timestamp: no_timestamp_sql,
    417            exp.TimestampTrunc: timestamptrunc_sql,
    418            exp.TimeStrToDate: timestrtotime_sql,
    419            exp.TimeStrToTime: timestrtotime_sql,
    420            exp.TimeStrToUnix: lambda self, e: self.func(
    421                "TO_UNIXTIME", self.func("DATE_PARSE", e.this, Presto.TIME_FORMAT)
    422            ),
    423            exp.TimeToStr: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    424            exp.TimeToUnix: rename_func("TO_UNIXTIME"),
    425            exp.ToChar: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    426            exp.TryCast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    427            exp.TsOrDiToDi: lambda self,
    428            e: f"CAST(SUBSTR(REPLACE(CAST({self.sql(e, 'this')} AS VARCHAR), '-', ''), 1, 8) AS INT)",
    429            exp.TsOrDsAdd: _ts_or_ds_add_sql,
    430            exp.TsOrDsDiff: _ts_or_ds_diff_sql,
    431            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    432            exp.Unhex: rename_func("FROM_HEX"),
    433            exp.UnixToStr: lambda self,
    434            e: f"DATE_FORMAT(FROM_UNIXTIME({self.sql(e, 'this')}), {self.format_time(e)})",
    435            exp.UnixToTime: _unix_to_time_sql,
    436            exp.UnixToTimeStr: lambda self,
    437            e: f"CAST(FROM_UNIXTIME({self.sql(e, 'this')}) AS VARCHAR)",
    438            exp.VariancePop: rename_func("VAR_POP"),
    439            exp.With: transforms.preprocess([transforms.add_recursive_cte_column_names]),
    440            exp.WithinGroup: transforms.preprocess(
    441                [transforms.remove_within_group_for_percentiles]
    442            ),
    443            exp.Xor: bool_xor_sql,
    444        }
    445
    446        RESERVED_KEYWORDS = {
    447            "alter",
    448            "and",
    449            "as",
    450            "between",
    451            "by",
    452            "case",
    453            "cast",
    454            "constraint",
    455            "create",
    456            "cross",
    457            "current_time",
    458            "current_timestamp",
    459            "deallocate",
    460            "delete",
    461            "describe",
    462            "distinct",
    463            "drop",
    464            "else",
    465            "end",
    466            "escape",
    467            "except",
    468            "execute",
    469            "exists",
    470            "extract",
    471            "false",
    472            "for",
    473            "from",
    474            "full",
    475            "group",
    476            "having",
    477            "in",
    478            "inner",
    479            "insert",
    480            "intersect",
    481            "into",
    482            "is",
    483            "join",
    484            "left",
    485            "like",
    486            "natural",
    487            "not",
    488            "null",
    489            "on",
    490            "or",
    491            "order",
    492            "outer",
    493            "prepare",
    494            "right",
    495            "select",
    496            "table",
    497            "then",
    498            "true",
    499            "union",
    500            "using",
    501            "values",
    502            "when",
    503            "where",
    504            "with",
    505        }
    506
    507        def strtounix_sql(self, expression: exp.StrToUnix) -> str:
    508            # Since `TO_UNIXTIME` requires a `TIMESTAMP`, we need to parse the argument into one.
    509            # To do this, we first try to `DATE_PARSE` it, but since this can fail when there's a
    510            # timezone involved, we wrap it in a `TRY` call and use `PARSE_DATETIME` as a fallback,
    511            # which seems to be using the same time mapping as Hive, as per:
    512            # https://joda-time.sourceforge.net/apidocs/org/joda/time/format/DateTimeFormat.html
    513            value_as_text = exp.cast(expression.this, exp.DataType.Type.TEXT)
    514            parse_without_tz = self.func("DATE_PARSE", value_as_text, self.format_time(expression))
    515            parse_with_tz = self.func(
    516                "PARSE_DATETIME",
    517                value_as_text,
    518                self.format_time(expression, Hive.INVERSE_TIME_MAPPING, Hive.INVERSE_TIME_TRIE),
    519            )
    520            coalesced = self.func("COALESCE", self.func("TRY", parse_without_tz), parse_with_tz)
    521            return self.func("TO_UNIXTIME", coalesced)
    522
    523        def bracket_sql(self, expression: exp.Bracket) -> str:
    524            if expression.args.get("safe"):
    525                return self.func(
    526                    "ELEMENT_AT",
    527                    expression.this,
    528                    seq_get(
    529                        apply_index_offset(
    530                            expression.this,
    531                            expression.expressions,
    532                            1 - expression.args.get("offset", 0),
    533                        ),
    534                        0,
    535                    ),
    536                )
    537            return super().bracket_sql(expression)
    538
    539        def struct_sql(self, expression: exp.Struct) -> str:
    540            from sqlglot.optimizer.annotate_types import annotate_types
    541
    542            expression = annotate_types(expression)
    543            values: t.List[str] = []
    544            schema: t.List[str] = []
    545            unknown_type = False
    546
    547            for e in expression.expressions:
    548                if isinstance(e, exp.PropertyEQ):
    549                    if e.type and e.type.is_type(exp.DataType.Type.UNKNOWN):
    550                        unknown_type = True
    551                    else:
    552                        schema.append(f"{self.sql(e, 'this')} {self.sql(e.type)}")
    553                    values.append(self.sql(e, "expression"))
    554                else:
    555                    values.append(self.sql(e))
    556
    557            size = len(expression.expressions)
    558
    559            if not size or len(schema) != size:
    560                if unknown_type:
    561                    self.unsupported(
    562                        "Cannot convert untyped key-value definitions (try annotate_types)."
    563                    )
    564                return self.func("ROW", *values)
    565            return f"CAST(ROW({', '.join(values)}) AS ROW({', '.join(schema)}))"
    566
    567        def interval_sql(self, expression: exp.Interval) -> str:
    568            if expression.this and expression.text("unit").upper().startswith("WEEK"):
    569                return f"({expression.this.name} * INTERVAL '7' DAY)"
    570            return super().interval_sql(expression)
    571
    572        def transaction_sql(self, expression: exp.Transaction) -> str:
    573            modes = expression.args.get("modes")
    574            modes = f" {', '.join(modes)}" if modes else ""
    575            return f"START TRANSACTION{modes}"
    576
    577        def generateseries_sql(self, expression: exp.GenerateSeries) -> str:
    578            start = expression.args["start"]
    579            end = expression.args["end"]
    580            step = expression.args.get("step")
    581
    582            if isinstance(start, exp.Cast):
    583                target_type = start.to
    584            elif isinstance(end, exp.Cast):
    585                target_type = end.to
    586            else:
    587                target_type = None
    588
    589            if target_type and target_type.is_type("timestamp"):
    590                if target_type is start.to:
    591                    end = exp.cast(end, target_type)
    592                else:
    593                    start = exp.cast(start, target_type)
    594
    595            return self.func("SEQUENCE", start, end, step)
    596
    597        def offset_limit_modifiers(
    598            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    599        ) -> t.List[str]:
    600            return [
    601                self.sql(expression, "offset"),
    602                self.sql(limit),
    603            ]
    604
    605        def create_sql(self, expression: exp.Create) -> str:
    606            """
    607            Presto doesn't support CREATE VIEW with expressions (ex: `CREATE VIEW x (cola)` then `(cola)` is the expression),
    608            so we need to remove them
    609            """
    610            kind = expression.args["kind"]
    611            schema = expression.this
    612            if kind == "VIEW" and schema.expressions:
    613                expression.this.set("expressions", None)
    614            return super().create_sql(expression)

class Presto(sqlglot.dialects.dialect.Dialect): View Source

    196class Presto(Dialect):
    197    INDEX_OFFSET = 1
    198    NULL_ORDERING = "nulls_are_last"
    199    TIME_FORMAT = MySQL.TIME_FORMAT
    200    TIME_MAPPING = MySQL.TIME_MAPPING
    201    STRICT_STRING_CONCAT = True
    202    SUPPORTS_SEMI_ANTI_JOIN = False
    203    TYPED_DIVISION = True
    204    TABLESAMPLE_SIZE_IS_PERCENT = True
    205    LOG_BASE_FIRST: t.Optional[bool] = None
    206
    207    # https://github.com/trinodb/trino/issues/17
    208    # https://github.com/trinodb/trino/issues/12289
    209    # https://github.com/prestodb/presto/issues/2863
    210    NORMALIZATION_STRATEGY = NormalizationStrategy.CASE_INSENSITIVE
    211
    212    class Tokenizer(tokens.Tokenizer):
    213        UNICODE_STRINGS = [
    214            (prefix + q, q)
    215            for q in t.cast(t.List[str], tokens.Tokenizer.QUOTES)
    216            for prefix in ("U&", "u&")
    217        ]
    218
    219        KEYWORDS = {
    220            **tokens.Tokenizer.KEYWORDS,
    221            "START": TokenType.BEGIN,
    222            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    223            "ROW": TokenType.STRUCT,
    224            "IPADDRESS": TokenType.IPADDRESS,
    225            "IPPREFIX": TokenType.IPPREFIX,
    226        }
    227
    228        KEYWORDS.pop("QUALIFY")
    229
    230    class Parser(parser.Parser):
    231        VALUES_FOLLOWED_BY_PAREN = False
    232
    233        FUNCTIONS = {
    234            **parser.Parser.FUNCTIONS,
    235            "ARBITRARY": exp.AnyValue.from_arg_list,
    236            "APPROX_DISTINCT": exp.ApproxDistinct.from_arg_list,
    237            "APPROX_PERCENTILE": _build_approx_percentile,
    238            "BITWISE_AND": binary_from_function(exp.BitwiseAnd),
    239            "BITWISE_NOT": lambda args: exp.BitwiseNot(this=seq_get(args, 0)),
    240            "BITWISE_OR": binary_from_function(exp.BitwiseOr),
    241            "BITWISE_XOR": binary_from_function(exp.BitwiseXor),
    242            "CARDINALITY": exp.ArraySize.from_arg_list,
    243            "CONTAINS": exp.ArrayContains.from_arg_list,
    244            "DATE_ADD": lambda args: exp.DateAdd(
    245                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    246            ),
    247            "DATE_DIFF": lambda args: exp.DateDiff(
    248                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    249            ),
    250            "DATE_FORMAT": build_formatted_time(exp.TimeToStr, "presto"),
    251            "DATE_PARSE": build_formatted_time(exp.StrToTime, "presto"),
    252            "DATE_TRUNC": date_trunc_to_time,
    253            "ELEMENT_AT": lambda args: exp.Bracket(
    254                this=seq_get(args, 0), expressions=[seq_get(args, 1)], offset=1, safe=True
    255            ),
    256            "FROM_HEX": exp.Unhex.from_arg_list,
    257            "FROM_UNIXTIME": _build_from_unixtime,
    258            "FROM_UTF8": lambda args: exp.Decode(
    259                this=seq_get(args, 0), replace=seq_get(args, 1), charset=exp.Literal.string("utf-8")
    260            ),
    261            "NOW": exp.CurrentTimestamp.from_arg_list,
    262            "REGEXP_EXTRACT": lambda args: exp.RegexpExtract(
    263                this=seq_get(args, 0), expression=seq_get(args, 1), group=seq_get(args, 2)
    264            ),
    265            "REGEXP_REPLACE": lambda args: exp.RegexpReplace(
    266                this=seq_get(args, 0),
    267                expression=seq_get(args, 1),
    268                replacement=seq_get(args, 2) or exp.Literal.string(""),
    269            ),
    270            "ROW": exp.Struct.from_arg_list,
    271            "SEQUENCE": exp.GenerateSeries.from_arg_list,
    272            "SET_AGG": exp.ArrayUniqueAgg.from_arg_list,
    273            "SPLIT_TO_MAP": exp.StrToMap.from_arg_list,
    274            "STRPOS": lambda args: exp.StrPosition(
    275                this=seq_get(args, 0), substr=seq_get(args, 1), instance=seq_get(args, 2)
    276            ),
    277            "TO_CHAR": _build_to_char,
    278            "TO_HEX": exp.Hex.from_arg_list,
    279            "TO_UNIXTIME": exp.TimeToUnix.from_arg_list,
    280            "TO_UTF8": lambda args: exp.Encode(
    281                this=seq_get(args, 0), charset=exp.Literal.string("utf-8")
    282            ),
    283        }
    284
    285        FUNCTION_PARSERS = parser.Parser.FUNCTION_PARSERS.copy()
    286        FUNCTION_PARSERS.pop("TRIM")
    287
    288    class Generator(generator.Generator):
    289        INTERVAL_ALLOWS_PLURAL_FORM = False
    290        JOIN_HINTS = False
    291        TABLE_HINTS = False
    292        QUERY_HINTS = False
    293        IS_BOOL_ALLOWED = False
    294        TZ_TO_WITH_TIME_ZONE = True
    295        NVL2_SUPPORTED = False
    296        STRUCT_DELIMITER = ("(", ")")
    297        LIMIT_ONLY_LITERALS = True
    298        SUPPORTS_SINGLE_ARG_CONCAT = False
    299        LIKE_PROPERTY_INSIDE_SCHEMA = True
    300        MULTI_ARG_DISTINCT = False
    301        SUPPORTS_TO_NUMBER = False
    302
    303        PROPERTIES_LOCATION = {
    304            **generator.Generator.PROPERTIES_LOCATION,
    305            exp.LocationProperty: exp.Properties.Location.UNSUPPORTED,
    306            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    307        }
    308
    309        TYPE_MAPPING = {
    310            **generator.Generator.TYPE_MAPPING,
    311            exp.DataType.Type.INT: "INTEGER",
    312            exp.DataType.Type.FLOAT: "REAL",
    313            exp.DataType.Type.BINARY: "VARBINARY",
    314            exp.DataType.Type.TEXT: "VARCHAR",
    315            exp.DataType.Type.TIMETZ: "TIME",
    316            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    317            exp.DataType.Type.STRUCT: "ROW",
    318            exp.DataType.Type.DATETIME: "TIMESTAMP",
    319            exp.DataType.Type.DATETIME64: "TIMESTAMP",
    320        }
    321
    322        TRANSFORMS = {
    323            **generator.Generator.TRANSFORMS,
    324            exp.AnyValue: rename_func("ARBITRARY"),
    325            exp.ApproxDistinct: lambda self, e: self.func(
    326                "APPROX_DISTINCT", e.this, e.args.get("accuracy")
    327            ),
    328            exp.ApproxQuantile: rename_func("APPROX_PERCENTILE"),
    329            exp.ArgMax: rename_func("MAX_BY"),
    330            exp.ArgMin: rename_func("MIN_BY"),
    331            exp.Array: lambda self, e: f"ARRAY[{self.expressions(e, flat=True)}]",
    332            exp.ArrayAny: rename_func("ANY_MATCH"),
    333            exp.ArrayConcat: rename_func("CONCAT"),
    334            exp.ArrayContains: rename_func("CONTAINS"),
    335            exp.ArraySize: rename_func("CARDINALITY"),
    336            exp.ArrayToString: rename_func("ARRAY_JOIN"),
    337            exp.ArrayUniqueAgg: rename_func("SET_AGG"),
    338            exp.AtTimeZone: rename_func("AT_TIMEZONE"),
    339            exp.BitwiseAnd: lambda self, e: self.func("BITWISE_AND", e.this, e.expression),
    340            exp.BitwiseLeftShift: lambda self, e: self.func(
    341                "BITWISE_ARITHMETIC_SHIFT_LEFT", e.this, e.expression
    342            ),
    343            exp.BitwiseNot: lambda self, e: self.func("BITWISE_NOT", e.this),
    344            exp.BitwiseOr: lambda self, e: self.func("BITWISE_OR", e.this, e.expression),
    345            exp.BitwiseRightShift: lambda self, e: self.func(
    346                "BITWISE_ARITHMETIC_SHIFT_RIGHT", e.this, e.expression
    347            ),
    348            exp.BitwiseXor: lambda self, e: self.func("BITWISE_XOR", e.this, e.expression),
    349            exp.Cast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    350            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    351            exp.DateAdd: lambda self, e: self.func(
    352                "DATE_ADD",
    353                unit_to_str(e),
    354                _to_int(e.expression),
    355                e.this,
    356            ),
    357            exp.DateDiff: lambda self, e: self.func(
    358                "DATE_DIFF", unit_to_str(e), e.expression, e.this
    359            ),
    360            exp.DateStrToDate: datestrtodate_sql,
    361            exp.DateToDi: lambda self,
    362            e: f"CAST(DATE_FORMAT({self.sql(e, 'this')}, {Presto.DATEINT_FORMAT}) AS INT)",
    363            exp.DateSub: lambda self, e: self.func(
    364                "DATE_ADD",
    365                unit_to_str(e),
    366                _to_int(e.expression * -1),
    367                e.this,
    368            ),
    369            exp.Decode: lambda self, e: encode_decode_sql(self, e, "FROM_UTF8"),
    370            exp.DiToDate: lambda self,
    371            e: f"CAST(DATE_PARSE(CAST({self.sql(e, 'this')} AS VARCHAR), {Presto.DATEINT_FORMAT}) AS DATE)",
    372            exp.Encode: lambda self, e: encode_decode_sql(self, e, "TO_UTF8"),
    373            exp.FileFormatProperty: lambda self, e: f"FORMAT='{e.name.upper()}'",
    374            exp.First: _first_last_sql,
    375            exp.FirstValue: _first_last_sql,
    376            exp.FromTimeZone: lambda self,
    377            e: f"WITH_TIMEZONE({self.sql(e, 'this')}, {self.sql(e, 'zone')}) AT TIME ZONE 'UTC'",
    378            exp.Group: transforms.preprocess([transforms.unalias_group]),
    379            exp.GroupConcat: lambda self, e: self.func(
    380                "ARRAY_JOIN", self.func("ARRAY_AGG", e.this), e.args.get("separator")
    381            ),
    382            exp.Hex: rename_func("TO_HEX"),
    383            exp.If: if_sql(),
    384            exp.ILike: no_ilike_sql,
    385            exp.Initcap: _initcap_sql,
    386            exp.ParseJSON: rename_func("JSON_PARSE"),
    387            exp.Last: _first_last_sql,
    388            exp.LastValue: _first_last_sql,
    389            exp.LastDay: lambda self, e: self.func("LAST_DAY_OF_MONTH", e.this),
    390            exp.Lateral: _explode_to_unnest_sql,
    391            exp.Left: left_to_substring_sql,
    392            exp.Levenshtein: rename_func("LEVENSHTEIN_DISTANCE"),
    393            exp.LogicalAnd: rename_func("BOOL_AND"),
    394            exp.LogicalOr: rename_func("BOOL_OR"),
    395            exp.Pivot: no_pivot_sql,
    396            exp.Quantile: _quantile_sql,
    397            exp.RegexpExtract: regexp_extract_sql,
    398            exp.Right: right_to_substring_sql,
    399            exp.SafeDivide: no_safe_divide_sql,
    400            exp.Schema: _schema_sql,
    401            exp.SchemaCommentProperty: lambda self, e: self.naked_property(e),
    402            exp.Select: transforms.preprocess(
    403                [
    404                    transforms.eliminate_qualify,
    405                    transforms.eliminate_distinct_on,
    406                    transforms.explode_to_unnest(1),
    407                    transforms.eliminate_semi_and_anti_joins,
    408                ]
    409            ),
    410            exp.SortArray: _no_sort_array,
    411            exp.StrPosition: lambda self, e: str_position_sql(self, e, generate_instance=True),
    412            exp.StrToDate: lambda self, e: f"CAST({_str_to_time_sql(self, e)} AS DATE)",
    413            exp.StrToMap: rename_func("SPLIT_TO_MAP"),
    414            exp.StrToTime: _str_to_time_sql,
    415            exp.StructExtract: struct_extract_sql,
    416            exp.Table: transforms.preprocess([_unnest_sequence]),
    417            exp.Timestamp: no_timestamp_sql,
    418            exp.TimestampTrunc: timestamptrunc_sql,
    419            exp.TimeStrToDate: timestrtotime_sql,
    420            exp.TimeStrToTime: timestrtotime_sql,
    421            exp.TimeStrToUnix: lambda self, e: self.func(
    422                "TO_UNIXTIME", self.func("DATE_PARSE", e.this, Presto.TIME_FORMAT)
    423            ),
    424            exp.TimeToStr: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    425            exp.TimeToUnix: rename_func("TO_UNIXTIME"),
    426            exp.ToChar: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    427            exp.TryCast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    428            exp.TsOrDiToDi: lambda self,
    429            e: f"CAST(SUBSTR(REPLACE(CAST({self.sql(e, 'this')} AS VARCHAR), '-', ''), 1, 8) AS INT)",
    430            exp.TsOrDsAdd: _ts_or_ds_add_sql,
    431            exp.TsOrDsDiff: _ts_or_ds_diff_sql,
    432            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    433            exp.Unhex: rename_func("FROM_HEX"),
    434            exp.UnixToStr: lambda self,
    435            e: f"DATE_FORMAT(FROM_UNIXTIME({self.sql(e, 'this')}), {self.format_time(e)})",
    436            exp.UnixToTime: _unix_to_time_sql,
    437            exp.UnixToTimeStr: lambda self,
    438            e: f"CAST(FROM_UNIXTIME({self.sql(e, 'this')}) AS VARCHAR)",
    439            exp.VariancePop: rename_func("VAR_POP"),
    440            exp.With: transforms.preprocess([transforms.add_recursive_cte_column_names]),
    441            exp.WithinGroup: transforms.preprocess(
    442                [transforms.remove_within_group_for_percentiles]
    443            ),
    444            exp.Xor: bool_xor_sql,
    445        }
    446
    447        RESERVED_KEYWORDS = {
    448            "alter",
    449            "and",
    450            "as",
    451            "between",
    452            "by",
    453            "case",
    454            "cast",
    455            "constraint",
    456            "create",
    457            "cross",
    458            "current_time",
    459            "current_timestamp",
    460            "deallocate",
    461            "delete",
    462            "describe",
    463            "distinct",
    464            "drop",
    465            "else",
    466            "end",
    467            "escape",
    468            "except",
    469            "execute",
    470            "exists",
    471            "extract",
    472            "false",
    473            "for",
    474            "from",
    475            "full",
    476            "group",
    477            "having",
    478            "in",
    479            "inner",
    480            "insert",
    481            "intersect",
    482            "into",
    483            "is",
    484            "join",
    485            "left",
    486            "like",
    487            "natural",
    488            "not",
    489            "null",
    490            "on",
    491            "or",
    492            "order",
    493            "outer",
    494            "prepare",
    495            "right",
    496            "select",
    497            "table",
    498            "then",
    499            "true",
    500            "union",
    501            "using",
    502            "values",
    503            "when",
    504            "where",
    505            "with",
    506        }
    507
    508        def strtounix_sql(self, expression: exp.StrToUnix) -> str:
    509            # Since `TO_UNIXTIME` requires a `TIMESTAMP`, we need to parse the argument into one.
    510            # To do this, we first try to `DATE_PARSE` it, but since this can fail when there's a
    511            # timezone involved, we wrap it in a `TRY` call and use `PARSE_DATETIME` as a fallback,
    512            # which seems to be using the same time mapping as Hive, as per:
    513            # https://joda-time.sourceforge.net/apidocs/org/joda/time/format/DateTimeFormat.html
    514            value_as_text = exp.cast(expression.this, exp.DataType.Type.TEXT)
    515            parse_without_tz = self.func("DATE_PARSE", value_as_text, self.format_time(expression))
    516            parse_with_tz = self.func(
    517                "PARSE_DATETIME",
    518                value_as_text,
    519                self.format_time(expression, Hive.INVERSE_TIME_MAPPING, Hive.INVERSE_TIME_TRIE),
    520            )
    521            coalesced = self.func("COALESCE", self.func("TRY", parse_without_tz), parse_with_tz)
    522            return self.func("TO_UNIXTIME", coalesced)
    523
    524        def bracket_sql(self, expression: exp.Bracket) -> str:
    525            if expression.args.get("safe"):
    526                return self.func(
    527                    "ELEMENT_AT",
    528                    expression.this,
    529                    seq_get(
    530                        apply_index_offset(
    531                            expression.this,
    532                            expression.expressions,
    533                            1 - expression.args.get("offset", 0),
    534                        ),
    535                        0,
    536                    ),
    537                )
    538            return super().bracket_sql(expression)
    539
    540        def struct_sql(self, expression: exp.Struct) -> str:
    541            from sqlglot.optimizer.annotate_types import annotate_types
    542
    543            expression = annotate_types(expression)
    544            values: t.List[str] = []
    545            schema: t.List[str] = []
    546            unknown_type = False
    547
    548            for e in expression.expressions:
    549                if isinstance(e, exp.PropertyEQ):
    550                    if e.type and e.type.is_type(exp.DataType.Type.UNKNOWN):
    551                        unknown_type = True
    552                    else:
    553                        schema.append(f"{self.sql(e, 'this')} {self.sql(e.type)}")
    554                    values.append(self.sql(e, "expression"))
    555                else:
    556                    values.append(self.sql(e))
    557
    558            size = len(expression.expressions)
    559
    560            if not size or len(schema) != size:
    561                if unknown_type:
    562                    self.unsupported(
    563                        "Cannot convert untyped key-value definitions (try annotate_types)."
    564                    )
    565                return self.func("ROW", *values)
    566            return f"CAST(ROW({', '.join(values)}) AS ROW({', '.join(schema)}))"
    567
    568        def interval_sql(self, expression: exp.Interval) -> str:
    569            if expression.this and expression.text("unit").upper().startswith("WEEK"):
    570                return f"({expression.this.name} * INTERVAL '7' DAY)"
    571            return super().interval_sql(expression)
    572
    573        def transaction_sql(self, expression: exp.Transaction) -> str:
    574            modes = expression.args.get("modes")
    575            modes = f" {', '.join(modes)}" if modes else ""
    576            return f"START TRANSACTION{modes}"
    577
    578        def generateseries_sql(self, expression: exp.GenerateSeries) -> str:
    579            start = expression.args["start"]
    580            end = expression.args["end"]
    581            step = expression.args.get("step")
    582
    583            if isinstance(start, exp.Cast):
    584                target_type = start.to
    585            elif isinstance(end, exp.Cast):
    586                target_type = end.to
    587            else:
    588                target_type = None
    589
    590            if target_type and target_type.is_type("timestamp"):
    591                if target_type is start.to:
    592                    end = exp.cast(end, target_type)
    593                else:
    594                    start = exp.cast(start, target_type)
    595
    596            return self.func("SEQUENCE", start, end, step)
    597
    598        def offset_limit_modifiers(
    599            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    600        ) -> t.List[str]:
    601            return [
    602                self.sql(expression, "offset"),
    603                self.sql(limit),
    604            ]
    605
    606        def create_sql(self, expression: exp.Create) -> str:
    607            """
    608            Presto doesn't support CREATE VIEW with expressions (ex: `CREATE VIEW x (cola)` then `(cola)` is the expression),
    609            so we need to remove them
    610            """
    611            kind = expression.args["kind"]
    612            schema = expression.this
    613            if kind == "VIEW" and schema.expressions:
    614                expression.this.set("expressions", None)
    615            return super().create_sql(expression)

INDEX_OFFSET = 1

The base index offset for arrays.

NULL_ORDERING = 'nulls_are_last'

Default `NULL` ordering method to use if not explicitly set. Possible values: `"nulls_are_small"`, `"nulls_are_large"`, `"nulls_are_last"`

TIME_FORMAT = "'%Y-%m-%d %T'"
TIME_MAPPING = {'%M': '%B', '%c': '%-m', '%e': '%-d', '%h': '%I', '%i': '%M', '%s': '%S', '%u': '%W', '%k': '%-H', '%l': '%-I', '%T': '%H:%M:%S', '%W': '%a'}

Associates this dialect's time formats with their equivalent Python `strftime` formats.

STRICT_STRING_CONCAT = True

Whether `CONCAT`'s arguments must be strings.

SUPPORTS_SEMI_ANTI_JOIN = False

Whether `SEMI` or `ANTI` joins are supported.

TYPED_DIVISION = True

Whether the behavior of `a / b` depends on the types of `a` and `b`. False means `a / b` is always float division. True means `a / b` is integer division if both `a` and `b` are integers.

TABLESAMPLE_SIZE_IS_PERCENT = True

Whether a size in the table sample clause represents percentage.

LOG_BASE_FIRST: Optional[bool] = None

Whether the base comes first in the `LOG` function. Possible values: `True`, `False`, `None` (two arguments are not supported by `LOG`)

NORMALIZATION_STRATEGY = <NormalizationStrategy.CASE_INSENSITIVE: 'CASE_INSENSITIVE'>

Specifies the strategy according to which identifiers should be normalized.

tokenizer_class = <class 'Presto.Tokenizer'>
parser_class = <class 'Presto.Parser'>
generator_class = <class 'Presto.Generator'>
TIME_TRIE: Dict = {'%': {'M': {0: True}, 'c': {0: True}, 'e': {0: True}, 'h': {0: True}, 'i': {0: True}, 's': {0: True}, 'u': {0: True}, 'k': {0: True}, 'l': {0: True}, 'T': {0: True}, 'W': {0: True}}}
FORMAT_TRIE: Dict = {'%': {'M': {0: True}, 'c': {0: True}, 'e': {0: True}, 'h': {0: True}, 'i': {0: True}, 's': {0: True}, 'u': {0: True}, 'k': {0: True}, 'l': {0: True}, 'T': {0: True}, 'W': {0: True}}}
INVERSE_TIME_MAPPING: Dict[str, str] = {'%B': '%M', '%-m': '%c', '%-d': '%e', '%I': '%h', '%M': '%i', '%S': '%s', '%W': '%u', '%-H': '%k', '%-I': '%l', '%H:%M:%S': '%T', '%a': '%W'}
INVERSE_TIME_TRIE: Dict = {'%': {'B': {0: True}, '-': {'m': {0: True}, 'd': {0: True}, 'H': {0: True}, 'I': {0: True}}, 'I': {0: True}, 'M': {0: True}, 'S': {0: True}, 'W': {0: True}, 'H': {':': {'%': {'M': {':': {'%': {'S': {0: True}}}}}}}, 'a': {0: True}}}
ESCAPED_SEQUENCES: Dict[str, str] = {}
QUOTE_START = "'"
QUOTE_END = "'"
IDENTIFIER_START = '"'
IDENTIFIER_END = '"'
BIT_START: Optional[str] = None
BIT_END: Optional[str] = None
HEX_START: Optional[str] = None
HEX_END: Optional[str] = None
BYTE_START: Optional[str] = None
BYTE_END: Optional[str] = None
UNICODE_START: Optional[str] = "U&'"
UNICODE_END: Optional[str] = "'"

##### Inherited Members

sqlglot.dialects.dialect.Dialect
  Dialect
  WEEK_OFFSET
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  IDENTIFIERS_CAN_START_WITH_DIGIT
  DPIPE_IS_STRING_CONCAT
  SUPPORTS_USER_DEFINED_TYPES
  NORMALIZE_FUNCTIONS
  SAFE_DIVISION
  CONCAT_COALESCE
  DATE_FORMAT
  DATEINT_FORMAT
  FORMAT_MAPPING
  UNESCAPED_SEQUENCES
  PSEUDOCOLUMNS
  PREFER_CTE_ALIAS_COLUMN
  get_or_raise
  format_time
  normalize_identifier
  case_sensitive
  can_identify
  quote_identifier
  to_json_path
  parse
  parse_into
  generate
  transpile
  tokenize
  tokenizer
  parser
  generator

class Presto.Tokenizer(sqlglot.tokens.Tokenizer): View Source

    212    class Tokenizer(tokens.Tokenizer):
    213        UNICODE_STRINGS = [
    214            (prefix + q, q)
    215            for q in t.cast(t.List[str], tokens.Tokenizer.QUOTES)
    216            for prefix in ("U&", "u&")
    217        ]
    218
    219        KEYWORDS = {
    220            **tokens.Tokenizer.KEYWORDS,
    221            "START": TokenType.BEGIN,
    222            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    223            "ROW": TokenType.STRUCT,
    224            "IPADDRESS": TokenType.IPADDRESS,
    225            "IPPREFIX": TokenType.IPPREFIX,
    226        }
    227
    228        KEYWORDS.pop("QUALIFY")

UNICODE_STRINGS = [("U&'", "'"), ("u&'", "'")]
KEYWORDS = {'{%': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%-': <TokenType.BLOCK_START: 'BLOCK_START'>, '%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '+%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '{{+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{{-': <TokenType.BLOCK_START: 'BLOCK_START'>, '+}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '/*+': <TokenType.HINT: 'HINT'>, '==': <TokenType.EQ: 'EQ'>, '::': <TokenType.DCOLON: 'DCOLON'>, '||': <TokenType.DPIPE: 'DPIPE'>, '>=': <TokenType.GTE: 'GTE'>, '<=': <TokenType.LTE: 'LTE'>, '<>': <TokenType.NEQ: 'NEQ'>, '!=': <TokenType.NEQ: 'NEQ'>, ':=': <TokenType.COLON_EQ: 'COLON_EQ'>, '<=>': <TokenType.NULLSAFE_EQ: 'NULLSAFE_EQ'>, '->': <TokenType.ARROW: 'ARROW'>, '->>': <TokenType.DARROW: 'DARROW'>, '=>': <TokenType.FARROW: 'FARROW'>, '#>': <TokenType.HASH_ARROW: 'HASH_ARROW'>, '#>>': <TokenType.DHASH_ARROW: 'DHASH_ARROW'>, '<->': <TokenType.LR_ARROW: 'LR_ARROW'>, '&&': <TokenType.DAMP: 'DAMP'>, '??': <TokenType.DQMARK: 'DQMARK'>, 'ALL': <TokenType.ALL: 'ALL'>, 'ALWAYS': <TokenType.ALWAYS: 'ALWAYS'>, 'AND': <TokenType.AND: 'AND'>, 'ANTI': <TokenType.ANTI: 'ANTI'>, 'ANY': <TokenType.ANY: 'ANY'>, 'ASC': <TokenType.ASC: 'ASC'>, 'AS': <TokenType.ALIAS: 'ALIAS'>, 'ASOF': <TokenType.ASOF: 'ASOF'>, 'AUTOINCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'AUTO_INCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'BEGIN': <TokenType.BEGIN: 'BEGIN'>, 'BETWEEN': <TokenType.BETWEEN: 'BETWEEN'>, 'CACHE': <TokenType.CACHE: 'CACHE'>, 'UNCACHE': <TokenType.UNCACHE: 'UNCACHE'>, 'CASE': <TokenType.CASE: 'CASE'>, 'CHARACTER SET': <TokenType.CHARACTER_SET: 'CHARACTER_SET'>, 'CLUSTER BY': <TokenType.CLUSTER_BY: 'CLUSTER_BY'>, 'COLLATE': <TokenType.COLLATE: 'COLLATE'>, 'COLUMN': <TokenType.COLUMN: 'COLUMN'>, 'COMMIT': <TokenType.COMMIT: 'COMMIT'>, 'CONNECT BY': <TokenType.CONNECT_BY: 'CONNECT_BY'>, 'CONSTRAINT': <TokenType.CONSTRAINT: 'CONSTRAINT'>, 'CREATE': <TokenType.CREATE: 'CREATE'>, 'CROSS': <TokenType.CROSS: 'CROSS'>, 'CUBE': <TokenType.CUBE: 'CUBE'>, 'CURRENT_DATE': <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, 'CURRENT_TIME': <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, 'CURRENT_TIMESTAMP': <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, 'CURRENT_USER': <TokenType.CURRENT_USER: 'CURRENT_USER'>, 'DATABASE': <TokenType.DATABASE: 'DATABASE'>, 'DEFAULT': <TokenType.DEFAULT: 'DEFAULT'>, 'DELETE': <TokenType.DELETE: 'DELETE'>, 'DESC': <TokenType.DESC: 'DESC'>, 'DESCRIBE': <TokenType.DESCRIBE: 'DESCRIBE'>, 'DISTINCT': <TokenType.DISTINCT: 'DISTINCT'>, 'DISTRIBUTE BY': <TokenType.DISTRIBUTE_BY: 'DISTRIBUTE_BY'>, 'DIV': <TokenType.DIV: 'DIV'>, 'DROP': <TokenType.DROP: 'DROP'>, 'ELSE': <TokenType.ELSE: 'ELSE'>, 'END': <TokenType.END: 'END'>, 'ENUM': <TokenType.ENUM: 'ENUM'>, 'ESCAPE': <TokenType.ESCAPE: 'ESCAPE'>, 'EXCEPT': <TokenType.EXCEPT: 'EXCEPT'>, 'EXECUTE': <TokenType.EXECUTE: 'EXECUTE'>, 'EXISTS': <TokenType.EXISTS: 'EXISTS'>, 'FALSE': <TokenType.FALSE: 'FALSE'>, 'FETCH': <TokenType.FETCH: 'FETCH'>, 'FILTER': <TokenType.FILTER: 'FILTER'>, 'FIRST': <TokenType.FIRST: 'FIRST'>, 'FULL': <TokenType.FULL: 'FULL'>, 'FUNCTION': <TokenType.FUNCTION: 'FUNCTION'>, 'FOR': <TokenType.FOR: 'FOR'>, 'FOREIGN KEY': <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, 'FORMAT': <TokenType.FORMAT: 'FORMAT'>, 'FROM': <TokenType.FROM: 'FROM'>, 'GEOGRAPHY': <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, 'GEOMETRY': <TokenType.GEOMETRY: 'GEOMETRY'>, 'GLOB': <TokenType.GLOB: 'GLOB'>, 'GROUP BY': <TokenType.GROUP_BY: 'GROUP_BY'>, 'GROUPING SETS': <TokenType.GROUPING_SETS: 'GROUPING_SETS'>, 'HAVING': <TokenType.HAVING: 'HAVING'>, 'ILIKE': <TokenType.ILIKE: 'ILIKE'>, 'IN': <TokenType.IN: 'IN'>, 'INDEX': <TokenType.INDEX: 'INDEX'>, 'INET': <TokenType.INET: 'INET'>, 'INNER': <TokenType.INNER: 'INNER'>, 'INSERT': <TokenType.INSERT: 'INSERT'>, 'INTERVAL': <TokenType.INTERVAL: 'INTERVAL'>, 'INTERSECT': <TokenType.INTERSECT: 'INTERSECT'>, 'INTO': <TokenType.INTO: 'INTO'>, 'IS': <TokenType.IS: 'IS'>, 'ISNULL': <TokenType.ISNULL: 'ISNULL'>, 'JOIN': <TokenType.JOIN: 'JOIN'>, 'KEEP': <TokenType.KEEP: 'KEEP'>, 'KILL': <TokenType.KILL: 'KILL'>, 'LATERAL': <TokenType.LATERAL: 'LATERAL'>, 'LEFT': <TokenType.LEFT: 'LEFT'>, 'LIKE': <TokenType.LIKE: 'LIKE'>, 'LIMIT': <TokenType.LIMIT: 'LIMIT'>, 'LOAD': <TokenType.LOAD: 'LOAD'>, 'LOCK': <TokenType.LOCK: 'LOCK'>, 'MERGE': <TokenType.MERGE: 'MERGE'>, 'NATURAL': <TokenType.NATURAL: 'NATURAL'>, 'NEXT': <TokenType.NEXT: 'NEXT'>, 'NOT': <TokenType.NOT: 'NOT'>, 'NOTNULL': <TokenType.NOTNULL: 'NOTNULL'>, 'NULL': <TokenType.NULL: 'NULL'>, 'OBJECT': <TokenType.OBJECT: 'OBJECT'>, 'OFFSET': <TokenType.OFFSET: 'OFFSET'>, 'ON': <TokenType.ON: 'ON'>, 'OR': <TokenType.OR: 'OR'>, 'XOR': <TokenType.XOR: 'XOR'>, 'ORDER BY': <TokenType.ORDER_BY: 'ORDER_BY'>, 'ORDINALITY': <TokenType.ORDINALITY: 'ORDINALITY'>, 'OUTER': <TokenType.OUTER: 'OUTER'>, 'OVER': <TokenType.OVER: 'OVER'>, 'OVERLAPS': <TokenType.OVERLAPS: 'OVERLAPS'>, 'OVERWRITE': <TokenType.OVERWRITE: 'OVERWRITE'>, 'PARTITION': <TokenType.PARTITION: 'PARTITION'>, 'PARTITION BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED_BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PERCENT': <TokenType.PERCENT: 'PERCENT'>, 'PIVOT': <TokenType.PIVOT: 'PIVOT'>, 'PRAGMA': <TokenType.PRAGMA: 'PRAGMA'>, 'PRIMARY KEY': <TokenType.PRIMARY_KEY: 'PRIMARY_KEY'>, 'PROCEDURE': <TokenType.PROCEDURE: 'PROCEDURE'>, 'RANGE': <TokenType.RANGE: 'RANGE'>, 'RECURSIVE': <TokenType.RECURSIVE: 'RECURSIVE'>, 'REGEXP': <TokenType.RLIKE: 'RLIKE'>, 'REPLACE': <TokenType.REPLACE: 'REPLACE'>, 'RETURNING': <TokenType.RETURNING: 'RETURNING'>, 'REFERENCES': <TokenType.REFERENCES: 'REFERENCES'>, 'RIGHT': <TokenType.RIGHT: 'RIGHT'>, 'RLIKE': <TokenType.RLIKE: 'RLIKE'>, 'ROLLBACK': <TokenType.ROLLBACK: 'ROLLBACK'>, 'ROLLUP': <TokenType.ROLLUP: 'ROLLUP'>, 'ROW': <TokenType.STRUCT: 'STRUCT'>, 'ROWS': <TokenType.ROWS: 'ROWS'>, 'SCHEMA': <TokenType.SCHEMA: 'SCHEMA'>, 'SELECT': <TokenType.SELECT: 'SELECT'>, 'SEMI': <TokenType.SEMI: 'SEMI'>, 'SET': <TokenType.SET: 'SET'>, 'SETTINGS': <TokenType.SETTINGS: 'SETTINGS'>, 'SHOW': <TokenType.SHOW: 'SHOW'>, 'SIMILAR TO': <TokenType.SIMILAR_TO: 'SIMILAR_TO'>, 'SOME': <TokenType.SOME: 'SOME'>, 'SORT BY': <TokenType.SORT_BY: 'SORT_BY'>, 'START WITH': <TokenType.START_WITH: 'START_WITH'>, 'TABLE': <TokenType.TABLE: 'TABLE'>, 'TABLESAMPLE': <TokenType.TABLE_SAMPLE: 'TABLE_SAMPLE'>, 'TEMP': <TokenType.TEMPORARY: 'TEMPORARY'>, 'TEMPORARY': <TokenType.TEMPORARY: 'TEMPORARY'>, 'THEN': <TokenType.THEN: 'THEN'>, 'TRUE': <TokenType.TRUE: 'TRUE'>, 'TRUNCATE': <TokenType.TRUNCATE: 'TRUNCATE'>, 'UNION': <TokenType.UNION: 'UNION'>, 'UNKNOWN': <TokenType.UNKNOWN: 'UNKNOWN'>, 'UNNEST': <TokenType.UNNEST: 'UNNEST'>, 'UNPIVOT': <TokenType.UNPIVOT: 'UNPIVOT'>, 'UPDATE': <TokenType.UPDATE: 'UPDATE'>, 'USE': <TokenType.USE: 'USE'>, 'USING': <TokenType.USING: 'USING'>, 'UUID': <TokenType.UUID: 'UUID'>, 'VALUES': <TokenType.VALUES: 'VALUES'>, 'VIEW': <TokenType.VIEW: 'VIEW'>, 'VOLATILE': <TokenType.VOLATILE: 'VOLATILE'>, 'WHEN': <TokenType.WHEN: 'WHEN'>, 'WHERE': <TokenType.WHERE: 'WHERE'>, 'WINDOW': <TokenType.WINDOW: 'WINDOW'>, 'WITH': <TokenType.WITH: 'WITH'>, 'APPLY': <TokenType.APPLY: 'APPLY'>, 'ARRAY': <TokenType.ARRAY: 'ARRAY'>, 'BIT': <TokenType.BIT: 'BIT'>, 'BOOL': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BOOLEAN': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BYTE': <TokenType.TINYINT: 'TINYINT'>, 'MEDIUMINT': <TokenType.MEDIUMINT: 'MEDIUMINT'>, 'INT1': <TokenType.TINYINT: 'TINYINT'>, 'TINYINT': <TokenType.TINYINT: 'TINYINT'>, 'INT16': <TokenType.SMALLINT: 'SMALLINT'>, 'SHORT': <TokenType.SMALLINT: 'SMALLINT'>, 'SMALLINT': <TokenType.SMALLINT: 'SMALLINT'>, 'INT128': <TokenType.INT128: 'INT128'>, 'HUGEINT': <TokenType.INT128: 'INT128'>, 'INT2': <TokenType.SMALLINT: 'SMALLINT'>, 'INTEGER': <TokenType.INT: 'INT'>, 'INT': <TokenType.INT: 'INT'>, 'INT4': <TokenType.INT: 'INT'>, 'INT32': <TokenType.INT: 'INT'>, 'INT64': <TokenType.BIGINT: 'BIGINT'>, 'LONG': <TokenType.BIGINT: 'BIGINT'>, 'BIGINT': <TokenType.BIGINT: 'BIGINT'>, 'INT8': <TokenType.TINYINT: 'TINYINT'>, 'UINT': <TokenType.UINT: 'UINT'>, 'DEC': <TokenType.DECIMAL: 'DECIMAL'>, 'DECIMAL': <TokenType.DECIMAL: 'DECIMAL'>, 'BIGDECIMAL': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'BIGNUMERIC': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'MAP': <TokenType.MAP: 'MAP'>, 'NULLABLE': <TokenType.NULLABLE: 'NULLABLE'>, 'NUMBER': <TokenType.DECIMAL: 'DECIMAL'>, 'NUMERIC': <TokenType.DECIMAL: 'DECIMAL'>, 'FIXED': <TokenType.DECIMAL: 'DECIMAL'>, 'REAL': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT4': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT8': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE PRECISION': <TokenType.DOUBLE: 'DOUBLE'>, 'JSON': <TokenType.JSON: 'JSON'>, 'CHAR': <TokenType.CHAR: 'CHAR'>, 'CHARACTER': <TokenType.CHAR: 'CHAR'>, 'NCHAR': <TokenType.NCHAR: 'NCHAR'>, 'VARCHAR': <TokenType.VARCHAR: 'VARCHAR'>, 'VARCHAR2': <TokenType.VARCHAR: 'VARCHAR'>, 'NVARCHAR': <TokenType.NVARCHAR: 'NVARCHAR'>, 'NVARCHAR2': <TokenType.NVARCHAR: 'NVARCHAR'>, 'BPCHAR': <TokenType.BPCHAR: 'BPCHAR'>, 'STR': <TokenType.TEXT: 'TEXT'>, 'STRING': <TokenType.TEXT: 'TEXT'>, 'TEXT': <TokenType.TEXT: 'TEXT'>, 'LONGTEXT': <TokenType.LONGTEXT: 'LONGTEXT'>, 'MEDIUMTEXT': <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, 'TINYTEXT': <TokenType.TINYTEXT: 'TINYTEXT'>, 'CLOB': <TokenType.TEXT: 'TEXT'>, 'LONGVARCHAR': <TokenType.TEXT: 'TEXT'>, 'BINARY': <TokenType.BINARY: 'BINARY'>, 'BLOB': <TokenType.VARBINARY: 'VARBINARY'>, 'LONGBLOB': <TokenType.LONGBLOB: 'LONGBLOB'>, 'MEDIUMBLOB': <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, 'TINYBLOB': <TokenType.TINYBLOB: 'TINYBLOB'>, 'BYTEA': <TokenType.VARBINARY: 'VARBINARY'>, 'VARBINARY': <TokenType.VARBINARY: 'VARBINARY'>, 'TIME': <TokenType.TIME: 'TIME'>, 'TIMETZ': <TokenType.TIMETZ: 'TIMETZ'>, 'TIMESTAMP': <TokenType.TIMESTAMP: 'TIMESTAMP'>, 'TIMESTAMPTZ': <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, 'TIMESTAMPLTZ': <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, 'DATE': <TokenType.DATE: 'DATE'>, 'DATETIME': <TokenType.DATETIME: 'DATETIME'>, 'INT4RANGE': <TokenType.INT4RANGE: 'INT4RANGE'>, 'INT4MULTIRANGE': <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, 'INT8RANGE': <TokenType.INT8RANGE: 'INT8RANGE'>, 'INT8MULTIRANGE': <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, 'NUMRANGE': <TokenType.NUMRANGE: 'NUMRANGE'>, 'NUMMULTIRANGE': <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, 'TSRANGE': <TokenType.TSRANGE: 'TSRANGE'>, 'TSMULTIRANGE': <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, 'TSTZRANGE': <TokenType.TSTZRANGE: 'TSTZRANGE'>, 'TSTZMULTIRANGE': <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, 'DATERANGE': <TokenType.DATERANGE: 'DATERANGE'>, 'DATEMULTIRANGE': <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, 'UNIQUE': <TokenType.UNIQUE: 'UNIQUE'>, 'STRUCT': <TokenType.STRUCT: 'STRUCT'>, 'SEQUENCE': <TokenType.SEQUENCE: 'SEQUENCE'>, 'VARIANT': <TokenType.VARIANT: 'VARIANT'>, 'ALTER': <TokenType.ALTER: 'ALTER'>, 'ANALYZE': <TokenType.COMMAND: 'COMMAND'>, 'CALL': <TokenType.COMMAND: 'COMMAND'>, 'COMMENT': <TokenType.COMMENT: 'COMMENT'>, 'COPY': <TokenType.COMMAND: 'COMMAND'>, 'EXPLAIN': <TokenType.COMMAND: 'COMMAND'>, 'GRANT': <TokenType.COMMAND: 'COMMAND'>, 'OPTIMIZE': <TokenType.COMMAND: 'COMMAND'>, 'PREPARE': <TokenType.COMMAND: 'COMMAND'>, 'VACUUM': <TokenType.COMMAND: 'COMMAND'>, 'USER-DEFINED': <TokenType.USERDEFINED: 'USERDEFINED'>, 'FOR VERSION': <TokenType.VERSION_SNAPSHOT: 'VERSION_SNAPSHOT'>, 'FOR TIMESTAMP': <TokenType.TIMESTAMP_SNAPSHOT: 'TIMESTAMP_SNAPSHOT'>, 'START': <TokenType.BEGIN: 'BEGIN'>, 'MATCH_RECOGNIZE': <TokenType.MATCH_RECOGNIZE: 'MATCH_RECOGNIZE'>, 'IPADDRESS': <TokenType.IPADDRESS: 'IPADDRESS'>, 'IPPREFIX': <TokenType.IPPREFIX: 'IPPREFIX'>}

##### Inherited Members

sqlglot.tokens.Tokenizer
  Tokenizer
  SINGLE_TOKENS
  BIT_STRINGS
  BYTE_STRINGS
  HEX_STRINGS
  RAW_STRINGS
  HEREDOC_STRINGS
  IDENTIFIERS
  IDENTIFIER_ESCAPES
  QUOTES
  STRING_ESCAPES
  VAR_SINGLE_TOKENS
  HEREDOC_TAG_IS_IDENTIFIER
  HEREDOC_STRING_ALTERNATIVE
  WHITE_SPACE
  COMMANDS
  COMMAND_PREFIX_TOKENS
  NUMERIC_LITERALS
  COMMENTS
  dialect
  reset
  tokenize
  tokenize_rs
  size
  sql
  tokens

class Presto.Parser(sqlglot.parser.Parser): View Source

    230    class Parser(parser.Parser):
    231        VALUES_FOLLOWED_BY_PAREN = False
    232
    233        FUNCTIONS = {
    234            **parser.Parser.FUNCTIONS,
    235            "ARBITRARY": exp.AnyValue.from_arg_list,
    236            "APPROX_DISTINCT": exp.ApproxDistinct.from_arg_list,
    237            "APPROX_PERCENTILE": _build_approx_percentile,
    238            "BITWISE_AND": binary_from_function(exp.BitwiseAnd),
    239            "BITWISE_NOT": lambda args: exp.BitwiseNot(this=seq_get(args, 0)),
    240            "BITWISE_OR": binary_from_function(exp.BitwiseOr),
    241            "BITWISE_XOR": binary_from_function(exp.BitwiseXor),
    242            "CARDINALITY": exp.ArraySize.from_arg_list,
    243            "CONTAINS": exp.ArrayContains.from_arg_list,
    244            "DATE_ADD": lambda args: exp.DateAdd(
    245                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    246            ),
    247            "DATE_DIFF": lambda args: exp.DateDiff(
    248                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    249            ),
    250            "DATE_FORMAT": build_formatted_time(exp.TimeToStr, "presto"),
    251            "DATE_PARSE": build_formatted_time(exp.StrToTime, "presto"),
    252            "DATE_TRUNC": date_trunc_to_time,
    253            "ELEMENT_AT": lambda args: exp.Bracket(
    254                this=seq_get(args, 0), expressions=[seq_get(args, 1)], offset=1, safe=True
    255            ),
    256            "FROM_HEX": exp.Unhex.from_arg_list,
    257            "FROM_UNIXTIME": _build_from_unixtime,
    258            "FROM_UTF8": lambda args: exp.Decode(
    259                this=seq_get(args, 0), replace=seq_get(args, 1), charset=exp.Literal.string("utf-8")
    260            ),
    261            "NOW": exp.CurrentTimestamp.from_arg_list,
    262            "REGEXP_EXTRACT": lambda args: exp.RegexpExtract(
    263                this=seq_get(args, 0), expression=seq_get(args, 1), group=seq_get(args, 2)
    264            ),
    265            "REGEXP_REPLACE": lambda args: exp.RegexpReplace(
    266                this=seq_get(args, 0),
    267                expression=seq_get(args, 1),
    268                replacement=seq_get(args, 2) or exp.Literal.string(""),
    269            ),
    270            "ROW": exp.Struct.from_arg_list,
    271            "SEQUENCE": exp.GenerateSeries.from_arg_list,
    272            "SET_AGG": exp.ArrayUniqueAgg.from_arg_list,
    273            "SPLIT_TO_MAP": exp.StrToMap.from_arg_list,
    274            "STRPOS": lambda args: exp.StrPosition(
    275                this=seq_get(args, 0), substr=seq_get(args, 1), instance=seq_get(args, 2)
    276            ),
    277            "TO_CHAR": _build_to_char,
    278            "TO_HEX": exp.Hex.from_arg_list,
    279            "TO_UNIXTIME": exp.TimeToUnix.from_arg_list,
    280            "TO_UTF8": lambda args: exp.Encode(
    281                this=seq_get(args, 0), charset=exp.Literal.string("utf-8")
    282            ),
    283        }
    284
    285        FUNCTION_PARSERS = parser.Parser.FUNCTION_PARSERS.copy()
    286        FUNCTION_PARSERS.pop("TRIM")

Parser consumes a list of tokens produced by the Tokenizer and produces a parsed syntax tree.

###### Arguments:

- error_level: The desired error level. Default: ErrorLevel.IMMEDIATE
- error_message_context: The amount of context to capture from a query string when displaying the error message (in number of characters). Default: 100
- max_errors: Maximum number of error messages to include in a raised ParseError. This is only relevant if error_level is ErrorLevel.RAISE. Default: 3

VALUES_FOLLOWED_BY_PAREN = False
FUNCTIONS = {'ABS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Abs'>>, 'ADD_MONTHS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AddMonths'>>, 'ANONYMOUS_AGG_FUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnonymousAggFunc'>>, 'ANY_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnyValue'>>, 'APPROX_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_COUNT_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxQuantile'>>, 'APPROX_TOP_K': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxTopK'>>, 'ARG_MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'ARGMAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'MAX_BY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'ARG_MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'ARGMIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'MIN_BY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Array'>>, 'ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAgg'>>, 'ARRAY_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAll'>>, 'ARRAY_ANY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAny'>>, 'ARRAY_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_OVERLAPS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayOverlaps'>>, 'ARRAY_SIZE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_SORT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySort'>>, 'ARRAY_SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySum'>>, 'ARRAY_TO_STRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayToString'>>, 'ARRAY_JOIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayToString'>>, 'ARRAY_UNION_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUnionAgg'>>, 'ARRAY_UNIQUE_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUniqueAgg'>>, 'AVG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Avg'>>, 'CASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Case'>>, 'CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cast'>>, 'CAST_TO_STR_TYPE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CastToStrType'>>, 'CBRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cbrt'>>, 'CEIL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CEILING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CHR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Chr'>>, 'CHAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Chr'>>, 'COALESCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'IFNULL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'NVL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'COLLATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Collate'>>, 'COMBINED_AGG_FUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CombinedAggFunc'>>, 'COMBINED_PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CombinedParameterizedAgg'>>, 'CONCAT': <function Parser.<lambda>>, 'CONCAT_WS': <function Parser.<lambda>>, 'CONNECT_BY_ROOT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ConnectByRoot'>>, 'CONVERT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Convert'>>, 'CORR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Corr'>>, 'COUNT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Count'>>, 'COUNT_IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'COUNTIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'COVAR_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CovarPop'>>, 'COVAR_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CovarSamp'>>, 'CURRENT_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDate'>>, 'CURRENT_DATETIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDatetime'>>, 'CURRENT_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTime'>>, 'CURRENT_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'CURRENT_USER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentUser'>>, 'DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Date'>>, 'DATE_ADD': <function Presto.Parser.<lambda>>, 'DATEDIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateDiff'>>, 'DATE_DIFF': <function Presto.Parser.<lambda>>, 'DATE_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATE_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateStrToDate'>>, 'DATE_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateSub'>>, 'DATE_TO_DATE_STR': <function Parser.<lambda>>, 'DATE_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateToDi'>>, 'DATE_TRUNC': <function date_trunc_to_time>, 'DATETIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeAdd'>>, 'DATETIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeDiff'>>, 'DATETIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeSub'>>, 'DATETIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeTrunc'>>, 'DAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Day'>>, 'DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAYOFMONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAY_OF_WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAYOFWEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAY_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DAYOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DECODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Decode'>>, 'DI_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DiToDate'>>, 'ENCODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Encode'>>, 'EXP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Exp'>>, 'EXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Explode'>>, 'EXPLODE_OUTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ExplodeOuter'>>, 'EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Extract'>>, 'FIRST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.First'>>, 'FIRST_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FirstValue'>>, 'FLATTEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Flatten'>>, 'FLOOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Floor'>>, 'FROM_BASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase'>>, 'FROM_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase64'>>, 'GENERATE_DATE_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateDateArray'>>, 'GENERATE_SERIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateSeries'>>, 'GREATEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Greatest'>>, 'GROUP_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GroupConcat'>>, 'HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hex'>>, 'HLL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hll'>>, 'IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'IIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'INITCAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Initcap'>>, 'IS_INF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsInf'>>, 'ISINF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsInf'>>, 'IS_NAN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsNan'>>, 'ISNAN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsNan'>>, 'J_S_O_N_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArray'>>, 'J_S_O_N_ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArrayAgg'>>, 'JSON_ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArrayContains'>>, 'JSONB_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtract'>>, 'JSONB_EXTRACT_SCALAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtractScalar'>>, 'JSON_EXTRACT': <function build_extract_json_with_path.<locals>._builder>, 'JSON_EXTRACT_SCALAR': <function build_extract_json_with_path.<locals>._builder>, 'JSON_FORMAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONFormat'>>, 'J_S_O_N_OBJECT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObject'>>, 'J_S_O_N_OBJECT_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObjectAgg'>>, 'J_S_O_N_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONTable'>>, 'LAG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lag'>>, 'LAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Last'>>, 'LAST_DAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDay'>>, 'LAST_DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDay'>>, 'LAST_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastValue'>>, 'LEAD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lead'>>, 'LEAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Least'>>, 'LEFT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Left'>>, 'LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEVENSHTEIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Levenshtein'>>, 'LN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ln'>>, 'LOG': <function build_logarithm>, 'LOGICAL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOLAND_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'LOGICAL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOLOR_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'LOWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'LCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'MD5': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5'>>, 'MD5_DIGEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5Digest'>>, 'MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Map'>>, 'MAP_FROM_ENTRIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MapFromEntries'>>, 'MATCH_AGAINST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MatchAgainst'>>, 'MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Max'>>, 'MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Min'>>, 'MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Month'>>, 'MONTHS_BETWEEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MonthsBetween'>>, 'NEXT_VALUE_FOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NextValueFor'>>, 'NTH_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NthValue'>>, 'NULLIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nullif'>>, 'NUMBER_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NumberToStr'>>, 'NVL2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nvl2'>>, 'OPEN_J_S_O_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.OpenJSON'>>, 'PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParameterizedAgg'>>, 'PARSE_JSON': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParseJSON'>>, 'JSON_PARSE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParseJSON'>>, 'PERCENTILE_CONT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileCont'>>, 'PERCENTILE_DISC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileDisc'>>, 'POSEXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Posexplode'>>, 'POSEXPLODE_OUTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PosexplodeOuter'>>, 'POWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'POW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'PREDICT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Predict'>>, 'QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Quantile'>>, 'QUARTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Quarter'>>, 'RAND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Rand'>>, 'RANDOM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Rand'>>, 'RANDN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Randn'>>, 'RANGE_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RangeN'>>, 'READ_CSV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ReadCSV'>>, 'REDUCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Reduce'>>, 'REGEXP_EXTRACT': <function Presto.Parser.<lambda>>, 'REGEXP_I_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpILike'>>, 'REGEXP_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpLike'>>, 'REGEXP_REPLACE': <function Presto.Parser.<lambda>>, 'REGEXP_SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpSplit'>>, 'REPEAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Repeat'>>, 'RIGHT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Right'>>, 'ROUND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Round'>>, 'ROW_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RowNumber'>>, 'SHA': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA1': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA2'>>, 'SAFE_DIVIDE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SafeDivide'>>, 'SIGN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sign'>>, 'SIGNUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sign'>>, 'SORT_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SortArray'>>, 'SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Split'>>, 'SQRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sqrt'>>, 'STANDARD_HASH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StandardHash'>>, 'STAR_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StarMap'>>, 'STARTS_WITH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StartsWith'>>, 'STARTSWITH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StartsWith'>>, 'STDDEV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stddev'>>, 'STDDEV_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevPop'>>, 'STDDEV_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevSamp'>>, 'STR_POSITION': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrPosition'>>, 'STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToDate'>>, 'STR_TO_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToMap'>>, 'STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToTime'>>, 'STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToUnix'>>, 'STRUCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Struct'>>, 'STRUCT_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StructExtract'>>, 'STUFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stuff'>>, 'INSERT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stuff'>>, 'SUBSTRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Substring'>>, 'SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sum'>>, 'TIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeAdd'>>, 'TIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeDiff'>>, 'TIME_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeFromParts'>>, 'TIMEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeFromParts'>>, 'TIME_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToDate'>>, 'TIME_STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToTime'>>, 'TIME_STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToUnix'>>, 'TIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeSub'>>, 'TIME_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToStr'>>, 'TIME_TO_TIME_STR': <function Parser.<lambda>>, 'TIME_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToUnix'>>, 'TIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeTrunc'>>, 'TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Timestamp'>>, 'TIMESTAMP_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampAdd'>>, 'TIMESTAMPDIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampDiff'>>, 'TIMESTAMP_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampDiff'>>, 'TIMESTAMP_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampFromParts'>>, 'TIMESTAMPFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampFromParts'>>, 'TIMESTAMP_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampSub'>>, 'TIMESTAMP_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampTrunc'>>, 'TO_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToArray'>>, 'TO_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToBase64'>>, 'TO_CHAR': <function _build_to_char>, 'TO_DAYS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToDays'>>, 'TO_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToMap'>>, 'TO_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToNumber'>>, 'TRANSFORM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Transform'>>, 'TRIM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Trim'>>, 'TRY_CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TryCast'>>, 'TS_OR_DI_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDiToDi'>>, 'TS_OR_DS_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsAdd'>>, 'TS_OR_DS_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsDiff'>>, 'TS_OR_DS_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToDate'>>, 'TS_OR_DS_TO_DATE_STR': <function Parser.<lambda>>, 'TS_OR_DS_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToTime'>>, 'TS_OR_DS_TO_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToTimestamp'>>, 'UNHEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Unhex'>>, 'UNIX_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixDate'>>, 'UNIX_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToStr'>>, 'UNIX_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTime'>>, 'UNIX_TO_TIME_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTimeStr'>>, 'UPPER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'UCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'VAR_MAP': <function build_var_map>, 'VARIANCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VAR_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'VAR_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Week'>>, 'WEEK_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WEEKOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WHEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.When'>>, 'X_M_L_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.XMLTable'>>, 'XOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Xor'>>, 'YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Year'>>, 'GLOB': <function Parser.<lambda>>, 'JSON_EXTRACT_PATH_TEXT': <function build_extract_json_with_path.<locals>._builder>, 'LIKE': <function build_like>, 'LOG2': <function Parser.<lambda>>, 'LOG10': <function Parser.<lambda>>, 'MOD': <function Parser.<lambda>>, 'ARBITRARY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnyValue'>>, 'APPROX_PERCENTILE': <function _build_approx_percentile>, 'BITWISE_AND': <function binary_from_function.<locals>.<lambda>>, 'BITWISE_NOT': <function Presto.Parser.<lambda>>, 'BITWISE_OR': <function binary_from_function.<locals>.<lambda>>, 'BITWISE_XOR': <function binary_from_function.<locals>.<lambda>>, 'CARDINALITY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'DATE_FORMAT': <function build_formatted_time.<locals>._builder>, 'DATE_PARSE': <function build_formatted_time.<locals>._builder>, 'ELEMENT_AT': <function Presto.Parser.<lambda>>, 'FROM_HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Unhex'>>, 'FROM_UNIXTIME': <function _build_from_unixtime>, 'FROM_UTF8': <function Presto.Parser.<lambda>>, 'NOW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'ROW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Struct'>>, 'SEQUENCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateSeries'>>, 'SET_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUniqueAgg'>>, 'SPLIT_TO_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToMap'>>, 'STRPOS': <function Presto.Parser.<lambda>>, 'TO_HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hex'>>, 'TO_UNIXTIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToUnix'>>, 'TO_UTF8': <function Presto.Parser.<lambda>>}
FUNCTION_PARSERS = {'CAST': <function Parser.<lambda>>, 'CONVERT': <function Parser.<lambda>>, 'DECODE': <function Parser.<lambda>>, 'EXTRACT': <function Parser.<lambda>>, 'JSON_OBJECT': <function Parser.<lambda>>, 'JSON_OBJECTAGG': <function Parser.<lambda>>, 'JSON_TABLE': <function Parser.<lambda>>, 'MATCH': <function Parser.<lambda>>, 'OPENJSON': <function Parser.<lambda>>, 'POSITION': <function Parser.<lambda>>, 'PREDICT': <function Parser.<lambda>>, 'SAFE_CAST': <function Parser.<lambda>>, 'STRING_AGG': <function Parser.<lambda>>, 'SUBSTRING': <function Parser.<lambda>>, 'TRY_CAST': <function Parser.<lambda>>, 'TRY_CONVERT': <function Parser.<lambda>>}
TABLE_ALIAS_TOKENS = {<TokenType.IPADDRESS: 'IPADDRESS'>, <TokenType.IPV4: 'IPV4'>, <TokenType.DATABASE: 'DATABASE'>, <TokenType.TOP: 'TOP'>, <TokenType.INT8RANGE: 'INT8RANGE'>, <TokenType.USE: 'USE'>, <TokenType.ROWS: 'ROWS'>, <TokenType.UNIQUEIDENTIFIER: 'UNIQUEIDENTIFIER'>, <TokenType.CURRENT_DATETIME: 'CURRENT_DATETIME'>, <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>, <TokenType.ENUM: 'ENUM'>, <TokenType.ARRAY: 'ARRAY'>, <TokenType.TIMETZ: 'TIMETZ'>, <TokenType.DELETE: 'DELETE'>, <TokenType.UBIGINT: 'UBIGINT'>, <TokenType.TIMESTAMP_NS: 'TIMESTAMP_NS'>, <TokenType.HLLSKETCH: 'HLLSKETCH'>, <TokenType.XML: 'XML'>, <TokenType.TEMPORARY: 'TEMPORARY'>, <TokenType.ANY: 'ANY'>, <TokenType.SMALLINT: 'SMALLINT'>, <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, <TokenType.BIT: 'BIT'>, <TokenType.MEDIUMINT: 'MEDIUMINT'>, <TokenType.CACHE: 'CACHE'>, <TokenType.IPPREFIX: 'IPPREFIX'>, <TokenType.NUMRANGE: 'NUMRANGE'>, <TokenType.END: 'END'>, <TokenType.BINARY: 'BINARY'>, <TokenType.FALSE: 'FALSE'>, <TokenType.MONEY: 'MONEY'>, <TokenType.SCHEMA: 'SCHEMA'>, <TokenType.COMMAND: 'COMMAND'>, <TokenType.NEXT: 'NEXT'>, <TokenType.STORAGE_INTEGRATION: 'STORAGE_INTEGRATION'>, <TokenType.NULL: 'NULL'>, <TokenType.ENUM8: 'ENUM8'>, <TokenType.CASE: 'CASE'>, <TokenType.PIVOT: 'PIVOT'>, <TokenType.NULLABLE: 'NULLABLE'>, <TokenType.LOAD: 'LOAD'>, <TokenType.DIV: 'DIV'>, <TokenType.PROCEDURE: 'PROCEDURE'>, <TokenType.UTINYINT: 'UTINYINT'>, <TokenType.UNKNOWN: 'UNKNOWN'>, <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, <TokenType.UMEDIUMINT: 'UMEDIUMINT'>, <TokenType.JSONB: 'JSONB'>, <TokenType.INET: 'INET'>, <TokenType.BIGINT: 'BIGINT'>, <TokenType.MERGE: 'MERGE'>, <TokenType.DATERANGE: 'DATERANGE'>, <TokenType.FINAL: 'FINAL'>, <TokenType.MAP: 'MAP'>, <TokenType.COLUMN: 'COLUMN'>, <TokenType.TIME: 'TIME'>, <TokenType.TIMESTAMP_MS: 'TIMESTAMP_MS'>, <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, <TokenType.NESTED: 'NESTED'>, <TokenType.ESCAPE: 'ESCAPE'>, <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, <TokenType.BEGIN: 'BEGIN'>, <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, <TokenType.UNPIVOT: 'UNPIVOT'>, <TokenType.TINYINT: 'TINYINT'>, <TokenType.UINT: 'UINT'>, <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, <TokenType.FIXEDSTRING: 'FIXEDSTRING'>, <TokenType.PRAGMA: 'PRAGMA'>, <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, <TokenType.TRUE: 'TRUE'>, <TokenType.TIMESTAMP_S: 'TIMESTAMP_S'>, <TokenType.INT: 'INT'>, <TokenType.TEXT: 'TEXT'>, <TokenType.INT128: 'INT128'>, <TokenType.VAR: 'VAR'>, <TokenType.NAME: 'NAME'>, <TokenType.ANTI: 'ANTI'>, <TokenType.ENUM16: 'ENUM16'>, <TokenType.CHAR: 'CHAR'>, <TokenType.SEQUENCE: 'SEQUENCE'>, <TokenType.UPDATE: 'UPDATE'>, <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, <TokenType.EXECUTE: 'EXECUTE'>, <TokenType.COLLATE: 'COLLATE'>, <TokenType.IPV6: 'IPV6'>, <TokenType.ROWVERSION: 'ROWVERSION'>, <TokenType.SEMI: 'SEMI'>, <TokenType.OVERWRITE: 'OVERWRITE'>, <TokenType.KEEP: 'KEEP'>, <TokenType.FIRST: 'FIRST'>, <TokenType.COMMIT: 'COMMIT'>, <TokenType.LONGBLOB: 'LONGBLOB'>, <TokenType.EXISTS: 'EXISTS'>, <TokenType.DECIMAL: 'DECIMAL'>, <TokenType.TINYTEXT: 'TINYTEXT'>, <TokenType.USERDEFINED: 'USERDEFINED'>, <TokenType.OBJECT: 'OBJECT'>, <TokenType.BOOLEAN: 'BOOLEAN'>, <TokenType.DATE: 'DATE'>, <TokenType.UDECIMAL: 'UDECIMAL'>, <TokenType.BIGSERIAL: 'BIGSERIAL'>, <TokenType.NVARCHAR: 'NVARCHAR'>, <TokenType.LONGTEXT: 'LONGTEXT'>, <TokenType.INT4RANGE: 'INT4RANGE'>, <TokenType.REFRESH: 'REFRESH'>, <TokenType.ORDINALITY: 'ORDINALITY'>, <TokenType.FILTER: 'FILTER'>, <TokenType.SIMPLEAGGREGATEFUNCTION: 'SIMPLEAGGREGATEFUNCTION'>, <TokenType.RECURSIVE: 'RECURSIVE'>, <TokenType.UUID: 'UUID'>, <TokenType.LOWCARDINALITY: 'LOWCARDINALITY'>, <TokenType.YEAR: 'YEAR'>, <TokenType.TSRANGE: 'TSRANGE'>, <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, <TokenType.COMMENT: 'COMMENT'>, <TokenType.REPLACE: 'REPLACE'>, <TokenType.USMALLINT: 'USMALLINT'>, <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, <TokenType.ROW: 'ROW'>, <TokenType.MODEL: 'MODEL'>, <TokenType.SUPER: 'SUPER'>, <TokenType.SMALLMONEY: 'SMALLMONEY'>, <TokenType.NCHAR: 'NCHAR'>, <TokenType.DEFAULT: 'DEFAULT'>, <TokenType.INTERVAL: 'INTERVAL'>, <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, <TokenType.TABLE: 'TABLE'>, <TokenType.DATETIME: 'DATETIME'>, <TokenType.DESCRIBE: 'DESCRIBE'>, <TokenType.PARTITION: 'PARTITION'>, <TokenType.AGGREGATEFUNCTION: 'AGGREGATEFUNCTION'>, <TokenType.UINT128: 'UINT128'>, <TokenType.TIMESTAMP: 'TIMESTAMP'>, <TokenType.TINYBLOB: 'TINYBLOB'>, <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, <TokenType.ISNULL: 'ISNULL'>, <TokenType.STRUCT: 'STRUCT'>, <TokenType.VOLATILE: 'VOLATILE'>, <TokenType.INT256: 'INT256'>, <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, <TokenType.UINT256: 'UINT256'>, <TokenType.ASC: 'ASC'>, <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, <TokenType.IDENTIFIER: 'IDENTIFIER'>, <TokenType.REFERENCES: 'REFERENCES'>, <TokenType.RANGE: 'RANGE'>, <TokenType.BPCHAR: 'BPCHAR'>, <TokenType.GEOMETRY: 'GEOMETRY'>, <TokenType.INDEX: 'INDEX'>, <TokenType.CURRENT_USER: 'CURRENT_USER'>, <TokenType.OPERATOR: 'OPERATOR'>, <TokenType.IS: 'IS'>, <TokenType.CONSTRAINT: 'CONSTRAINT'>, <TokenType.VARBINARY: 'VARBINARY'>, <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, <TokenType.HSTORE: 'HSTORE'>, <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, <TokenType.TSTZRANGE: 'TSTZRANGE'>, <TokenType.PERCENT: 'PERCENT'>, <TokenType.FUNCTION: 'FUNCTION'>, <TokenType.FLOAT: 'FLOAT'>, <TokenType.SETTINGS: 'SETTINGS'>, <TokenType.SOME: 'SOME'>, <TokenType.SERIAL: 'SERIAL'>, <TokenType.VIEW: 'VIEW'>, <TokenType.DICTIONARY: 'DICTIONARY'>, <TokenType.DESC: 'DESC'>, <TokenType.SHOW: 'SHOW'>, <TokenType.DOUBLE: 'DOUBLE'>, <TokenType.ALL: 'ALL'>, <TokenType.TRUNCATE: 'TRUNCATE'>, <TokenType.DATETIME64: 'DATETIME64'>, <TokenType.VARCHAR: 'VARCHAR'>, <TokenType.OVERLAPS: 'OVERLAPS'>, <TokenType.FORMAT: 'FORMAT'>, <TokenType.KILL: 'KILL'>, <TokenType.JSON: 'JSON'>, <TokenType.IMAGE: 'IMAGE'>, <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, <TokenType.VARIANT: 'VARIANT'>, <TokenType.DATE32: 'DATE32'>, <TokenType.UNIQUE: 'UNIQUE'>, <TokenType.SET: 'SET'>}
SHOW_TRIE: Dict = {}
SET_TRIE: Dict = {'GLOBAL': {0: True}, 'LOCAL': {0: True}, 'SESSION': {0: True}, 'TRANSACTION': {0: True}}

##### Inherited Members

sqlglot.parser.Parser
  Parser
  NO_PAREN_FUNCTIONS
  STRUCT_TYPE_TOKENS
  NESTED_TYPE_TOKENS
  ENUM_TYPE_TOKENS
  AGGREGATE_TYPE_TOKENS
  TYPE_TOKENS
  SIGNED_TO_UNSIGNED_TYPE_TOKEN
  SUBQUERY_PREDICATES
  RESERVED_TOKENS
  DB_CREATABLES
  CREATABLES
  ID_VAR_TOKENS
  INTERVAL_VARS
  ALIAS_TOKENS
  COMMENT_TABLE_ALIAS_TOKENS
  UPDATE_ALIAS_TOKENS
  TRIM_TYPES
  FUNC_TOKENS
  CONJUNCTION
  EQUALITY
  COMPARISON
  BITWISE
  TERM
  FACTOR
  EXPONENT
  TIMES
  TIMESTAMPS
  SET_OPERATIONS
  JOIN_METHODS
  JOIN_SIDES
  JOIN_KINDS
  JOIN_HINTS
  LAMBDAS
  COLUMN_OPERATORS
  EXPRESSION_PARSERS
  STATEMENT_PARSERS
  UNARY_PARSERS
  STRING_PARSERS
  NUMERIC_PARSERS
  PRIMARY_PARSERS
  PLACEHOLDER_PARSERS
  RANGE_PARSERS
  PROPERTY_PARSERS
  CONSTRAINT_PARSERS
  ALTER_PARSERS
  SCHEMA_UNNAMED_CONSTRAINTS
  NO_PAREN_FUNCTION_PARSERS
  INVALID_FUNC_NAME_TOKENS
  FUNCTIONS_WITH_ALIASED_ARGS
  KEY_VALUE_DEFINITIONS
  QUERY_MODIFIER_PARSERS
  SET_PARSERS
  SHOW_PARSERS
  TYPE_LITERAL_PARSERS
  DDL_SELECT_TOKENS
  PRE_VOLATILE_TOKENS
  TRANSACTION_KIND
  TRANSACTION_CHARACTERISTICS
  CONFLICT_ACTIONS
  CREATE_SEQUENCE
  ISOLATED_LOADING_OPTIONS
  USABLES
  CAST_ACTIONS
  INSERT_ALTERNATIVES
  CLONE_KEYWORDS
  HISTORICAL_DATA_KIND
  OPCLASS_FOLLOW_KEYWORDS
  OPTYPE_FOLLOW_TOKENS
  TABLE_INDEX_HINT_TOKENS
  VIEW_ATTRIBUTES
  WINDOW_ALIAS_TOKENS
  WINDOW_BEFORE_PAREN_TOKENS
  WINDOW_SIDES
  JSON_KEY_VALUE_SEPARATOR_TOKENS
  FETCH_TOKENS
  ADD_CONSTRAINT_TOKENS
  DISTINCT_TOKENS
  NULL_TOKENS
  UNNEST_OFFSET_ALIAS_TOKENS
  SELECT_START_TOKENS
  STRICT_CAST
  PREFIXED_PIVOT_COLUMNS
  IDENTIFY_PIVOT_STRINGS
  LOG_DEFAULTS_TO_LN
  ALTER_TABLE_ADD_REQUIRED_FOR_EACH_COLUMN
  TABLESAMPLE_CSV
  SET_REQUIRES_ASSIGNMENT_DELIMITER
  TRIM_PATTERN_FIRST
  STRING_ALIASES
  MODIFIERS_ATTACHED_TO_UNION
  UNION_MODIFIERS
  NO_PAREN_IF_COMMANDS
  JSON_ARROWS_REQUIRE_JSON_TYPE
  SUPPORTS_IMPLICIT_UNNEST
  INTERVAL_SPANS
  SUPPORTS_PARTITION_SELECTION
  error_level
  error_message_context
  max_errors
  dialect
  reset
  parse
  parse_into
  check_errors
  raise_error
  expression
  validate_expression
  errors
  sql

class Presto.Generator(sqlglot.generator.Generator): View Source

    288    class Generator(generator.Generator):
    289        INTERVAL_ALLOWS_PLURAL_FORM = False
    290        JOIN_HINTS = False
    291        TABLE_HINTS = False
    292        QUERY_HINTS = False
    293        IS_BOOL_ALLOWED = False
    294        TZ_TO_WITH_TIME_ZONE = True
    295        NVL2_SUPPORTED = False
    296        STRUCT_DELIMITER = ("(", ")")
    297        LIMIT_ONLY_LITERALS = True
    298        SUPPORTS_SINGLE_ARG_CONCAT = False
    299        LIKE_PROPERTY_INSIDE_SCHEMA = True
    300        MULTI_ARG_DISTINCT = False
    301        SUPPORTS_TO_NUMBER = False
    302
    303        PROPERTIES_LOCATION = {
    304            **generator.Generator.PROPERTIES_LOCATION,
    305            exp.LocationProperty: exp.Properties.Location.UNSUPPORTED,
    306            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    307        }
    308
    309        TYPE_MAPPING = {
    310            **generator.Generator.TYPE_MAPPING,
    311            exp.DataType.Type.INT: "INTEGER",
    312            exp.DataType.Type.FLOAT: "REAL",
    313            exp.DataType.Type.BINARY: "VARBINARY",
    314            exp.DataType.Type.TEXT: "VARCHAR",
    315            exp.DataType.Type.TIMETZ: "TIME",
    316            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    317            exp.DataType.Type.STRUCT: "ROW",
    318            exp.DataType.Type.DATETIME: "TIMESTAMP",
    319            exp.DataType.Type.DATETIME64: "TIMESTAMP",
    320        }
    321
    322        TRANSFORMS = {
    323            **generator.Generator.TRANSFORMS,
    324            exp.AnyValue: rename_func("ARBITRARY"),
    325            exp.ApproxDistinct: lambda self, e: self.func(
    326                "APPROX_DISTINCT", e.this, e.args.get("accuracy")
    327            ),
    328            exp.ApproxQuantile: rename_func("APPROX_PERCENTILE"),
    329            exp.ArgMax: rename_func("MAX_BY"),
    330            exp.ArgMin: rename_func("MIN_BY"),
    331            exp.Array: lambda self, e: f"ARRAY[{self.expressions(e, flat=True)}]",
    332            exp.ArrayAny: rename_func("ANY_MATCH"),
    333            exp.ArrayConcat: rename_func("CONCAT"),
    334            exp.ArrayContains: rename_func("CONTAINS"),
    335            exp.ArraySize: rename_func("CARDINALITY"),
    336            exp.ArrayToString: rename_func("ARRAY_JOIN"),
    337            exp.ArrayUniqueAgg: rename_func("SET_AGG"),
    338            exp.AtTimeZone: rename_func("AT_TIMEZONE"),
    339            exp.BitwiseAnd: lambda self, e: self.func("BITWISE_AND", e.this, e.expression),
    340            exp.BitwiseLeftShift: lambda self, e: self.func(
    341                "BITWISE_ARITHMETIC_SHIFT_LEFT", e.this, e.expression
    342            ),
    343            exp.BitwiseNot: lambda self, e: self.func("BITWISE_NOT", e.this),
    344            exp.BitwiseOr: lambda self, e: self.func("BITWISE_OR", e.this, e.expression),
    345            exp.BitwiseRightShift: lambda self, e: self.func(
    346                "BITWISE_ARITHMETIC_SHIFT_RIGHT", e.this, e.expression
    347            ),
    348            exp.BitwiseXor: lambda self, e: self.func("BITWISE_XOR", e.this, e.expression),
    349            exp.Cast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    350            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    351            exp.DateAdd: lambda self, e: self.func(
    352                "DATE_ADD",
    353                unit_to_str(e),
    354                _to_int(e.expression),
    355                e.this,
    356            ),
    357            exp.DateDiff: lambda self, e: self.func(
    358                "DATE_DIFF", unit_to_str(e), e.expression, e.this
    359            ),
    360            exp.DateStrToDate: datestrtodate_sql,
    361            exp.DateToDi: lambda self,
    362            e: f"CAST(DATE_FORMAT({self.sql(e, 'this')}, {Presto.DATEINT_FORMAT}) AS INT)",
    363            exp.DateSub: lambda self, e: self.func(
    364                "DATE_ADD",
    365                unit_to_str(e),
    366                _to_int(e.expression * -1),
    367                e.this,
    368            ),
    369            exp.Decode: lambda self, e: encode_decode_sql(self, e, "FROM_UTF8"),
    370            exp.DiToDate: lambda self,
    371            e: f"CAST(DATE_PARSE(CAST({self.sql(e, 'this')} AS VARCHAR), {Presto.DATEINT_FORMAT}) AS DATE)",
    372            exp.Encode: lambda self, e: encode_decode_sql(self, e, "TO_UTF8"),
    373            exp.FileFormatProperty: lambda self, e: f"FORMAT='{e.name.upper()}'",
    374            exp.First: _first_last_sql,
    375            exp.FirstValue: _first_last_sql,
    376            exp.FromTimeZone: lambda self,
    377            e: f"WITH_TIMEZONE({self.sql(e, 'this')}, {self.sql(e, 'zone')}) AT TIME ZONE 'UTC'",
    378            exp.Group: transforms.preprocess([transforms.unalias_group]),
    379            exp.GroupConcat: lambda self, e: self.func(
    380                "ARRAY_JOIN", self.func("ARRAY_AGG", e.this), e.args.get("separator")
    381            ),
    382            exp.Hex: rename_func("TO_HEX"),
    383            exp.If: if_sql(),
    384            exp.ILike: no_ilike_sql,
    385            exp.Initcap: _initcap_sql,
    386            exp.ParseJSON: rename_func("JSON_PARSE"),
    387            exp.Last: _first_last_sql,
    388            exp.LastValue: _first_last_sql,
    389            exp.LastDay: lambda self, e: self.func("LAST_DAY_OF_MONTH", e.this),
    390            exp.Lateral: _explode_to_unnest_sql,
    391            exp.Left: left_to_substring_sql,
    392            exp.Levenshtein: rename_func("LEVENSHTEIN_DISTANCE"),
    393            exp.LogicalAnd: rename_func("BOOL_AND"),
    394            exp.LogicalOr: rename_func("BOOL_OR"),
    395            exp.Pivot: no_pivot_sql,
    396            exp.Quantile: _quantile_sql,
    397            exp.RegexpExtract: regexp_extract_sql,
    398            exp.Right: right_to_substring_sql,
    399            exp.SafeDivide: no_safe_divide_sql,
    400            exp.Schema: _schema_sql,
    401            exp.SchemaCommentProperty: lambda self, e: self.naked_property(e),
    402            exp.Select: transforms.preprocess(
    403                [
    404                    transforms.eliminate_qualify,
    405                    transforms.eliminate_distinct_on,
    406                    transforms.explode_to_unnest(1),
    407                    transforms.eliminate_semi_and_anti_joins,
    408                ]
    409            ),
    410            exp.SortArray: _no_sort_array,
    411            exp.StrPosition: lambda self, e: str_position_sql(self, e, generate_instance=True),
    412            exp.StrToDate: lambda self, e: f"CAST({_str_to_time_sql(self, e)} AS DATE)",
    413            exp.StrToMap: rename_func("SPLIT_TO_MAP"),
    414            exp.StrToTime: _str_to_time_sql,
    415            exp.StructExtract: struct_extract_sql,
    416            exp.Table: transforms.preprocess([_unnest_sequence]),
    417            exp.Timestamp: no_timestamp_sql,
    418            exp.TimestampTrunc: timestamptrunc_sql,
    419            exp.TimeStrToDate: timestrtotime_sql,
    420            exp.TimeStrToTime: timestrtotime_sql,
    421            exp.TimeStrToUnix: lambda self, e: self.func(
    422                "TO_UNIXTIME", self.func("DATE_PARSE", e.this, Presto.TIME_FORMAT)
    423            ),
    424            exp.TimeToStr: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    425            exp.TimeToUnix: rename_func("TO_UNIXTIME"),
    426            exp.ToChar: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    427            exp.TryCast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    428            exp.TsOrDiToDi: lambda self,
    429            e: f"CAST(SUBSTR(REPLACE(CAST({self.sql(e, 'this')} AS VARCHAR), '-', ''), 1, 8) AS INT)",
    430            exp.TsOrDsAdd: _ts_or_ds_add_sql,
    431            exp.TsOrDsDiff: _ts_or_ds_diff_sql,
    432            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    433            exp.Unhex: rename_func("FROM_HEX"),
    434            exp.UnixToStr: lambda self,
    435            e: f"DATE_FORMAT(FROM_UNIXTIME({self.sql(e, 'this')}), {self.format_time(e)})",
    436            exp.UnixToTime: _unix_to_time_sql,
    437            exp.UnixToTimeStr: lambda self,
    438            e: f"CAST(FROM_UNIXTIME({self.sql(e, 'this')}) AS VARCHAR)",
    439            exp.VariancePop: rename_func("VAR_POP"),
    440            exp.With: transforms.preprocess([transforms.add_recursive_cte_column_names]),
    441            exp.WithinGroup: transforms.preprocess(
    442                [transforms.remove_within_group_for_percentiles]
    443            ),
    444            exp.Xor: bool_xor_sql,
    445        }
    446
    447        RESERVED_KEYWORDS = {
    448            "alter",
    449            "and",
    450            "as",
    451            "between",
    452            "by",
    453            "case",
    454            "cast",
    455            "constraint",
    456            "create",
    457            "cross",
    458            "current_time",
    459            "current_timestamp",
    460            "deallocate",
    461            "delete",
    462            "describe",
    463            "distinct",
    464            "drop",
    465            "else",
    466            "end",
    467            "escape",
    468            "except",
    469            "execute",
    470            "exists",
    471            "extract",
    472            "false",
    473            "for",
    474            "from",
    475            "full",
    476            "group",
    477            "having",
    478            "in",
    479            "inner",
    480            "insert",
    481            "intersect",
    482            "into",
    483            "is",
    484            "join",
    485            "left",
    486            "like",
    487            "natural",
    488            "not",
    489            "null",
    490            "on",
    491            "or",
    492            "order",
    493            "outer",
    494            "prepare",
    495            "right",
    496            "select",
    497            "table",
    498            "then",
    499            "true",
    500            "union",
    501            "using",
    502            "values",
    503            "when",
    504            "where",
    505            "with",
    506        }
    507
    508        def strtounix_sql(self, expression: exp.StrToUnix) -> str:
    509            # Since `TO_UNIXTIME` requires a `TIMESTAMP`, we need to parse the argument into one.
    510            # To do this, we first try to `DATE_PARSE` it, but since this can fail when there's a
    511            # timezone involved, we wrap it in a `TRY` call and use `PARSE_DATETIME` as a fallback,
    512            # which seems to be using the same time mapping as Hive, as per:
    513            # https://joda-time.sourceforge.net/apidocs/org/joda/time/format/DateTimeFormat.html
    514            value_as_text = exp.cast(expression.this, exp.DataType.Type.TEXT)
    515            parse_without_tz = self.func("DATE_PARSE", value_as_text, self.format_time(expression))
    516            parse_with_tz = self.func(
    517                "PARSE_DATETIME",
    518                value_as_text,
    519                self.format_time(expression, Hive.INVERSE_TIME_MAPPING, Hive.INVERSE_TIME_TRIE),
    520            )
    521            coalesced = self.func("COALESCE", self.func("TRY", parse_without_tz), parse_with_tz)
    522            return self.func("TO_UNIXTIME", coalesced)
    523
    524        def bracket_sql(self, expression: exp.Bracket) -> str:
    525            if expression.args.get("safe"):
    526                return self.func(
    527                    "ELEMENT_AT",
    528                    expression.this,
    529                    seq_get(
    530                        apply_index_offset(
    531                            expression.this,
    532                            expression.expressions,
    533                            1 - expression.args.get("offset", 0),
    534                        ),
    535                        0,
    536                    ),
    537                )
    538            return super().bracket_sql(expression)
    539
    540        def struct_sql(self, expression: exp.Struct) -> str:
    541            from sqlglot.optimizer.annotate_types import annotate_types
    542
    543            expression = annotate_types(expression)
    544            values: t.List[str] = []
    545            schema: t.List[str] = []
    546            unknown_type = False
    547
    548            for e in expression.expressions:
    549                if isinstance(e, exp.PropertyEQ):
    550                    if e.type and e.type.is_type(exp.DataType.Type.UNKNOWN):
    551                        unknown_type = True
    552                    else:
    553                        schema.append(f"{self.sql(e, 'this')} {self.sql(e.type)}")
    554                    values.append(self.sql(e, "expression"))
    555                else:
    556                    values.append(self.sql(e))
    557
    558            size = len(expression.expressions)
    559
    560            if not size or len(schema) != size:
    561                if unknown_type:
    562                    self.unsupported(
    563                        "Cannot convert untyped key-value definitions (try annotate_types)."
    564                    )
    565                return self.func("ROW", *values)
    566            return f"CAST(ROW({', '.join(values)}) AS ROW({', '.join(schema)}))"
    567
    568        def interval_sql(self, expression: exp.Interval) -> str:
    569            if expression.this and expression.text("unit").upper().startswith("WEEK"):
    570                return f"({expression.this.name} * INTERVAL '7' DAY)"
    571            return super().interval_sql(expression)
    572
    573        def transaction_sql(self, expression: exp.Transaction) -> str:
    574            modes = expression.args.get("modes")
    575            modes = f" {', '.join(modes)}" if modes else ""
    576            return f"START TRANSACTION{modes}"
    577
    578        def generateseries_sql(self, expression: exp.GenerateSeries) -> str:
    579            start = expression.args["start"]
    580            end = expression.args["end"]
    581            step = expression.args.get("step")
    582
    583            if isinstance(start, exp.Cast):
    584                target_type = start.to
    585            elif isinstance(end, exp.Cast):
    586                target_type = end.to
    587            else:
    588                target_type = None
    589
    590            if target_type and target_type.is_type("timestamp"):
    591                if target_type is start.to:
    592                    end = exp.cast(end, target_type)
    593                else:
    594                    start = exp.cast(start, target_type)
    595
    596            return self.func("SEQUENCE", start, end, step)
    597
    598        def offset_limit_modifiers(
    599            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    600        ) -> t.List[str]:
    601            return [
    602                self.sql(expression, "offset"),
    603                self.sql(limit),
    604            ]
    605
    606        def create_sql(self, expression: exp.Create) -> str:
    607            """
    608            Presto doesn't support CREATE VIEW with expressions (ex: `CREATE VIEW x (cola)` then `(cola)` is the expression),
    609            so we need to remove them
    610            """
    611            kind = expression.args["kind"]
    612            schema = expression.this
    613            if kind == "VIEW" and schema.expressions:
    614                expression.this.set("expressions", None)
    615            return super().create_sql(expression)

Generator converts a given syntax tree to the corresponding SQL string.

###### Arguments:

- pretty: Whether to format the produced SQL string. Default: False.
- identify: Determines when an identifier should be quoted. Possible values are: False (default): Never quote, except in cases where it's mandatory by the dialect. True or 'always': Always quote. 'safe': Only quote identifiers that are case insensitive.
- normalize: Whether to normalize identifiers to lowercase. Default: False.
- pad: The pad size in a formatted string. For example, this affects the indentation of a projection in a query, relative to its nesting level. Default: 2.
- indent: The indentation size in a formatted string. For example, this affects the indentation of subqueries and filters under a `WHERE` clause. Default: 2.
- normalize_functions: How to normalize function names. Possible values are: "upper" or True (default): Convert names to uppercase. "lower": Convert names to lowercase. False: Disables function name normalization.
- unsupported_level: Determines the generator's behavior when it encounters unsupported expressions. Default ErrorLevel.WARN.
- max_unsupported: Maximum number of unsupported messages to include in a raised UnsupportedError. This is only relevant if unsupported_level is ErrorLevel.RAISE. Default: 3
- leading_comma: Whether the comma is leading or trailing in select expressions. This is only relevant when generating in pretty mode. Default: False
- max_text_width: The max number of characters in a segment before creating new lines in pretty mode. The default is on the smaller end because the length only represents a segment and not the true line length. Default: 80
- comments: Whether to preserve comments in the output SQL code. Default: True

INTERVAL_ALLOWS_PLURAL_FORM = False
JOIN_HINTS = False
TABLE_HINTS = False
QUERY_HINTS = False
IS_BOOL_ALLOWED = False
TZ_TO_WITH_TIME_ZONE = True
NVL2_SUPPORTED = False
STRUCT_DELIMITER = ('(', ')')
LIMIT_ONLY_LITERALS = True
SUPPORTS_SINGLE_ARG_CONCAT = False
LIKE_PROPERTY_INSIDE_SCHEMA = True
MULTI_ARG_DISTINCT = False
SUPPORTS_TO_NUMBER = False
PROPERTIES_LOCATION = {<class 'sqlglot.expressions.AlgorithmProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.AutoIncrementProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.AutoRefreshProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BackupProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BlockCompressionProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CharacterSetProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ChecksumProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CollateProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Cluster'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ClusteredByProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DataBlocksizeProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.DefinerProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.DictRange'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DictProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistStyleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.EngineProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExternalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.FallbackProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.FileFormatProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.FreespaceProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.GlobalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.HeapProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.InheritsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.IcebergProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.InputModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.IsolatedLoadingProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.JournalProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.LanguageProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LikeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LocationProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.LockProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LockingProperty'>: <Location.POST_ALIAS: 'POST_ALIAS'>, <class 'sqlglot.expressions.LogProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.MaterializedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.MergeBlockRatioProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.OnProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.OnCommitProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.Order'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.OutputModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PartitionedByProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.PartitionedOfProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PrimaryKey'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Property'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.RemoteWithConnectionModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ReturnsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatDelimitedProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatSerdeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SampleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SchemaCommentProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SerdeProperties'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Set'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SettingsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SetProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.SetConfigProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SharingProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.SequenceProperties'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.SortKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlReadWriteProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.StabilityProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TemporaryProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.ToTableProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TransientProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.TransformModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.MergeTreeTTL'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.UnloggedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.ViewAttributeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.VolatileProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.WithDataProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.WithSystemVersioningProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>}
TYPE_MAPPING = {<Type.NCHAR: 'NCHAR'>: 'CHAR', <Type.NVARCHAR: 'NVARCHAR'>: 'VARCHAR', <Type.MEDIUMTEXT: 'MEDIUMTEXT'>: 'TEXT', <Type.LONGTEXT: 'LONGTEXT'>: 'TEXT', <Type.TINYTEXT: 'TINYTEXT'>: 'TEXT', <Type.MEDIUMBLOB: 'MEDIUMBLOB'>: 'BLOB', <Type.LONGBLOB: 'LONGBLOB'>: 'BLOB', <Type.TINYBLOB: 'TINYBLOB'>: 'BLOB', <Type.INET: 'INET'>: 'INET', <Type.INT: 'INT'>: 'INTEGER', <Type.FLOAT: 'FLOAT'>: 'REAL', <Type.BINARY: 'BINARY'>: 'VARBINARY', <Type.TEXT: 'TEXT'>: 'VARCHAR', <Type.TIMETZ: 'TIMETZ'>: 'TIME', <Type.TIMESTAMPTZ: 'TIMESTAMPTZ'>: 'TIMESTAMP', <Type.STRUCT: 'STRUCT'>: 'ROW', <Type.DATETIME: 'DATETIME'>: 'TIMESTAMP', <Type.DATETIME64: 'DATETIME64'>: 'TIMESTAMP'}
TRANSFORMS = {<class 'sqlglot.expressions.JSONPathFilter'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathKey'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathRecursive'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathRoot'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathScript'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathSelector'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathSlice'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathSubscript'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathUnion'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathWildcard'>: <function <lambda>>, <class 'sqlglot.expressions.AutoRefreshProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.BackupProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CaseSpecificColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ClusteredColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CollateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CommentColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DateFormatColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DefaultColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.EncodeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.EphemeralColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExcludeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExternalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.GlobalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.HeapProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.IcebergProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InheritsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InlineLengthColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InputModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.IntervalSpan'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.JSONExtract'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.JSONExtractScalar'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LanguageProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LocationProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LogProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.MaterializedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NonClusteredColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NotForReplicationColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnCommitProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnUpdateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OutputModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.PathColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.RemoteWithConnectionModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ReturnsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SampleProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetConfigProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SettingsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SharingProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlReadWriteProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.StabilityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TemporaryProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TitleColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.Timestamp'>: <function no_timestamp_sql>, <class 'sqlglot.expressions.ToMap'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ToTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransformModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransientProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UppercaseColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UnloggedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VarMap'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ViewAttributeProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VolatileProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithOperator'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.AnyValue'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ApproxDistinct'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.ApproxQuantile'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArgMax'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArgMin'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Array'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.ArrayAny'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayConcat'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayContains'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArraySize'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayToString'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayUniqueAgg'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.AtTimeZone'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.BitwiseAnd'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseLeftShift'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseNot'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseOr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseRightShift'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseXor'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Cast'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.CurrentTimestamp'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DateAdd'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DateDiff'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DateStrToDate'>: <function datestrtodate_sql>, <class 'sqlglot.expressions.DateToDi'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DateSub'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Decode'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DiToDate'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Encode'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.FileFormatProperty'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.First'>: <function _first_last_sql>, <class 'sqlglot.expressions.FirstValue'>: <function _first_last_sql>, <class 'sqlglot.expressions.FromTimeZone'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Group'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.GroupConcat'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Hex'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.If'>: <function if_sql.<locals>._if_sql>, <class 'sqlglot.expressions.ILike'>: <function no_ilike_sql>, <class 'sqlglot.expressions.Initcap'>: <function _initcap_sql>, <class 'sqlglot.expressions.ParseJSON'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Last'>: <function _first_last_sql>, <class 'sqlglot.expressions.LastValue'>: <function _first_last_sql>, <class 'sqlglot.expressions.LastDay'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Lateral'>: <function _explode_to_unnest_sql>, <class 'sqlglot.expressions.Left'>: <function left_to_substring_sql>, <class 'sqlglot.expressions.Levenshtein'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.LogicalAnd'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.LogicalOr'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Pivot'>: <function no_pivot_sql>, <class 'sqlglot.expressions.Quantile'>: <function _quantile_sql>, <class 'sqlglot.expressions.RegexpExtract'>: <function regexp_extract_sql>, <class 'sqlglot.expressions.Right'>: <function right_to_substring_sql>, <class 'sqlglot.expressions.SafeDivide'>: <function no_safe_divide_sql>, <class 'sqlglot.expressions.Schema'>: <function _schema_sql>, <class 'sqlglot.expressions.SchemaCommentProperty'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Select'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.SortArray'>: <function _no_sort_array>, <class 'sqlglot.expressions.StrPosition'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.StrToDate'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.StrToMap'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.StrToTime'>: <function _str_to_time_sql>, <class 'sqlglot.expressions.StructExtract'>: <function struct_extract_sql>, <class 'sqlglot.expressions.Table'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.TimestampTrunc'>: <function timestamptrunc_sql>, <class 'sqlglot.expressions.TimeStrToDate'>: <function timestrtotime_sql>, <class 'sqlglot.expressions.TimeStrToTime'>: <function timestrtotime_sql>, <class 'sqlglot.expressions.TimeStrToUnix'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TimeToStr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TimeToUnix'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ToChar'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TryCast'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.TsOrDiToDi'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TsOrDsAdd'>: <function _ts_or_ds_add_sql>, <class 'sqlglot.expressions.TsOrDsDiff'>: <function _ts_or_ds_diff_sql>, <class 'sqlglot.expressions.TsOrDsToDate'>: <function _ts_or_ds_to_date_sql>, <class 'sqlglot.expressions.Unhex'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.UnixToStr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.UnixToTime'>: <function _unix_to_time_sql>, <class 'sqlglot.expressions.UnixToTimeStr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.VariancePop'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.With'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.WithinGroup'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.Xor'>: <function bool_xor_sql>}
RESERVED_KEYWORDS = {'inner', 'true', 'current_time', 'alter', 'values', 'and', 'not', 'in', 'using', 'like', 'when', 'where', 'insert', 'order', 'on', 'between', 'cross', 'extract', 'execute', 'distinct', 'delete', 'except', 'with', 'table', 'cast', 'false', 'from', 'as', 'drop', 'then', 'natural', 'right', 'join', 'describe', 'exists', 'select', 'group', 'or', 'intersect', 'end', 'constraint', 'else', 'current_timestamp', 'left', 'full', 'create', 'for', 'by', 'escape', 'case', 'outer', 'null', 'prepare', 'into', 'deallocate', 'is', 'union', 'having'}
def strtounix_sql(self, expression: sqlglot.expressions.StrToUnix) -> str: View Source

    508        def strtounix_sql(self, expression: exp.StrToUnix) -> str:
    509            # Since `TO_UNIXTIME` requires a `TIMESTAMP`, we need to parse the argument into one.
    510            # To do this, we first try to `DATE_PARSE` it, but since this can fail when there's a
    511            # timezone involved, we wrap it in a `TRY` call and use `PARSE_DATETIME` as a fallback,
    512            # which seems to be using the same time mapping as Hive, as per:
    513            # https://joda-time.sourceforge.net/apidocs/org/joda/time/format/DateTimeFormat.html
    514            value_as_text = exp.cast(expression.this, exp.DataType.Type.TEXT)
    515            parse_without_tz = self.func("DATE_PARSE", value_as_text, self.format_time(expression))
    516            parse_with_tz = self.func(
    517                "PARSE_DATETIME",
    518                value_as_text,
    519                self.format_time(expression, Hive.INVERSE_TIME_MAPPING, Hive.INVERSE_TIME_TRIE),
    520            )
    521            coalesced = self.func("COALESCE", self.func("TRY", parse_without_tz), parse_with_tz)
    522            return self.func("TO_UNIXTIME", coalesced)

def bracket_sql(self, expression: sqlglot.expressions.Bracket) -> str: View Source

    524        def bracket_sql(self, expression: exp.Bracket) -> str:
    525            if expression.args.get("safe"):
    526                return self.func(
    527                    "ELEMENT_AT",
    528                    expression.this,
    529                    seq_get(
    530                        apply_index_offset(
    531                            expression.this,
    532                            expression.expressions,
    533                            1 - expression.args.get("offset", 0),
    534                        ),
    535                        0,
    536                    ),
    537                )
    538            return super().bracket_sql(expression)

def struct_sql(self, expression: sqlglot.expressions.Struct) -> str: View Source

    540        def struct_sql(self, expression: exp.Struct) -> str:
    541            from sqlglot.optimizer.annotate_types import annotate_types
    542
    543            expression = annotate_types(expression)
    544            values: t.List[str] = []
    545            schema: t.List[str] = []
    546            unknown_type = False
    547
    548            for e in expression.expressions:
    549                if isinstance(e, exp.PropertyEQ):
    550                    if e.type and e.type.is_type(exp.DataType.Type.UNKNOWN):
    551                        unknown_type = True
    552                    else:
    553                        schema.append(f"{self.sql(e, 'this')} {self.sql(e.type)}")
    554                    values.append(self.sql(e, "expression"))
    555                else:
    556                    values.append(self.sql(e))
    557
    558            size = len(expression.expressions)
    559
    560            if not size or len(schema) != size:
    561                if unknown_type:
    562                    self.unsupported(
    563                        "Cannot convert untyped key-value definitions (try annotate_types)."
    564                    )
    565                return self.func("ROW", *values)
    566            return f"CAST(ROW({', '.join(values)}) AS ROW({', '.join(schema)}))"

def interval_sql(self, expression: sqlglot.expressions.Interval) -> str: View Source

    568        def interval_sql(self, expression: exp.Interval) -> str:
    569            if expression.this and expression.text("unit").upper().startswith("WEEK"):
    570                return f"({expression.this.name} * INTERVAL '7' DAY)"
    571            return super().interval_sql(expression)

def transaction_sql(self, expression: sqlglot.expressions.Transaction) -> str: View Source

    573        def transaction_sql(self, expression: exp.Transaction) -> str:
    574            modes = expression.args.get("modes")
    575            modes = f" {', '.join(modes)}" if modes else ""
    576            return f"START TRANSACTION{modes}"

def generateseries_sql(self, expression: sqlglot.expressions.GenerateSeries) -> str: View Source

    578        def generateseries_sql(self, expression: exp.GenerateSeries) -> str:
    579            start = expression.args["start"]
    580            end = expression.args["end"]
    581            step = expression.args.get("step")
    582
    583            if isinstance(start, exp.Cast):
    584                target_type = start.to
    585            elif isinstance(end, exp.Cast):
    586                target_type = end.to
    587            else:
    588                target_type = None
    589
    590            if target_type and target_type.is_type("timestamp"):
    591                if target_type is start.to:
    592                    end = exp.cast(end, target_type)
    593                else:
    594                    start = exp.cast(start, target_type)
    595
    596            return self.func("SEQUENCE", start, end, step)

def offset_limit_modifiers( self, expression: sqlglot.expressions.Expression, fetch: bool, limit: Union[sqlglot.expressions.Fetch, sqlglot.expressions.Limit, NoneType]) -> List[str]: View Source

    598        def offset_limit_modifiers(
    599            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    600        ) -> t.List[str]:
    601            return [
    602                self.sql(expression, "offset"),
    603                self.sql(limit),
    604            ]

def create_sql(self, expression: sqlglot.expressions.Create) -> str: View Source

    606        def create_sql(self, expression: exp.Create) -> str:
    607            """
    608            Presto doesn't support CREATE VIEW with expressions (ex: `CREATE VIEW x (cola)` then `(cola)` is the expression),
    609            so we need to remove them
    610            """
    611            kind = expression.args["kind"]
    612            schema = expression.this
    613            if kind == "VIEW" and schema.expressions:
    614                expression.this.set("expressions", None)
    615            return super().create_sql(expression)

Presto doesn't support CREATE VIEW with expressions (ex: `CREATE VIEW x (cola)` then `(cola)` is the expression), so we need to remove them

SELECT_KINDS: Tuple[str, ...] = ()
AFTER_HAVING_MODIFIER_TRANSFORMS = {'qualify': <function Generator.<lambda>>, 'windows': <function Generator.<lambda>>}

##### Inherited Members

sqlglot.generator.Generator
  Generator
  NULL_ORDERING_SUPPORTED
  IGNORE_NULLS_IN_FUNC
  LOCKING_READS_SUPPORTED
  EXPLICIT_UNION
  WRAP_DERIVED_VALUES
  CREATE_FUNCTION_RETURN_AS
  MATCHED_BY_SOURCE
  SINGLE_STRING_INTERVAL
  LIMIT_FETCH
  RENAME_TABLE_WITH_DB
  GROUPINGS_SEP
  INDEX_ON
  QUERY_HINT_SEP
  DUPLICATE_KEY_UPDATE_WITH_SET
  LIMIT_IS_TOP
  RETURNING_END
  COLUMN_JOIN_MARKS_SUPPORTED
  EXTRACT_ALLOWS_QUOTES
  VALUES_AS_TABLE
  ALTER_TABLE_INCLUDE_COLUMN_KEYWORD
  UNNEST_WITH_ORDINALITY
  AGGREGATE_FILTER_SUPPORTED
  SEMI_ANTI_JOIN_WITH_SIDE
  COMPUTED_COLUMN_WITH_TYPE
  SUPPORTS_TABLE_COPY
  TABLESAMPLE_REQUIRES_PARENS
  TABLESAMPLE_SIZE_IS_ROWS
  TABLESAMPLE_KEYWORDS
  TABLESAMPLE_WITH_METHOD
  TABLESAMPLE_SEED_KEYWORD
  COLLATE_IS_FUNC
  DATA_TYPE_SPECIFIERS_ALLOWED
  ENSURE_BOOLS
  CTE_RECURSIVE_KEYWORD_REQUIRED
  LAST_DAY_SUPPORTS_DATE_PART
  SUPPORTS_TABLE_ALIAS_COLUMNS
  UNPIVOT_ALIASES_ARE_IDENTIFIERS
  JSON_KEY_VALUE_PAIR_SEP
  INSERT_OVERWRITE
  SUPPORTS_SELECT_INTO
  SUPPORTS_UNLOGGED_TABLES
  SUPPORTS_CREATE_TABLE_LIKE
  JSON_TYPE_REQUIRED_FOR_EXTRACTION
  JSON_PATH_BRACKETED_KEY_SUPPORTED
  JSON_PATH_SINGLE_QUOTE_ESCAPE
  SUPPORTED_JSON_PATH_PARTS
  CAN_IMPLEMENT_ARRAY_ANY
  OUTER_UNION_MODIFIERS
  STAR_MAPPING
  TIME_PART_SINGULARS
  TOKEN_MAPPING
  PARAMETER_TOKEN
  NAMED_PLACEHOLDER_TOKEN
  WITH_SEPARATED_COMMENTS
  EXCLUDE_COMMENTS
  UNWRAPPED_INTERVAL_VALUES
  PARAMETERIZABLE_TEXT_TYPES
  EXPRESSIONS_WITHOUT_NESTED_CTES
  SENTINEL_LINE_BREAK
  pretty
  identify
  normalize
  pad
  unsupported_level
  max_unsupported
  leading_comma
  max_text_width
  comments
  dialect
  normalize_functions
  unsupported_messages
  generate
  preprocess
  unsupported
  sep
  seg
  pad_comment
  maybe_comment
  wrap
  no_identify
  normalize_func
  indent
  sql
  uncache_sql
  cache_sql
  characterset_sql
  column_parts
  column_sql
  columnposition_sql
  columndef_sql
  columnconstraint_sql
  computedcolumnconstraint_sql
  autoincrementcolumnconstraint_sql
  compresscolumnconstraint_sql
  generatedasidentitycolumnconstraint_sql
  generatedasrowcolumnconstraint_sql
  periodforsystemtimeconstraint_sql
  notnullcolumnconstraint_sql
  transformcolumnconstraint_sql
  primarykeycolumnconstraint_sql
  uniquecolumnconstraint_sql
  createable_sql
  sequenceproperties_sql
  clone_sql
  describe_sql
  heredoc_sql
  prepend_ctes
  with_sql
  cte_sql
  tablealias_sql
  bitstring_sql
  hexstring_sql
  bytestring_sql
  unicodestring_sql
  rawstring_sql
  datatypeparam_sql
  datatype_sql
  directory_sql
  delete_sql
  drop_sql
  except_sql
  except_op
  fetch_sql
  filter_sql
  hint_sql
  indexparameters_sql
  index_sql
  identifier_sql
  inputoutputformat_sql
  national_sql
  partition_sql
  properties_sql
  root_properties
  properties
  with_properties
  locate_properties
  property_name
  property_sql
  likeproperty_sql
  fallbackproperty_sql
  journalproperty_sql
  freespaceproperty_sql
  checksumproperty_sql
  mergeblockratioproperty_sql
  datablocksizeproperty_sql
  blockcompressionproperty_sql
  isolatedloadingproperty_sql
  partitionboundspec_sql
  partitionedofproperty_sql
  lockingproperty_sql
  withdataproperty_sql
  withsystemversioningproperty_sql
  insert_sql
  intersect_sql
  intersect_op
  introducer_sql
  kill_sql
  pseudotype_sql
  objectidentifier_sql
  onconflict_sql
  returning_sql
  rowformatdelimitedproperty_sql
  withtablehint_sql
  indextablehint_sql
  historicaldata_sql
  table_parts
  table_sql
  tablesample_sql
  pivot_sql
  version_sql
  tuple_sql
  update_sql
  values_sql
  var_sql
  into_sql
  from_sql
  group_sql
  having_sql
  connect_sql
  prior_sql
  join_sql
  lambda_sql
  lateral_op
  lateral_sql
  limit_sql
  offset_sql
  setitem_sql
  set_sql
  pragma_sql
  lock_sql
  literal_sql
  escape_str
  loaddata_sql
  null_sql
  boolean_sql
  order_sql
  withfill_sql
  cluster_sql
  distribute_sql
  sort_sql
  ordered_sql
  matchrecognizemeasure_sql
  matchrecognize_sql
  query_modifiers
  queryoption_sql
  after_limit_modifiers
  select_sql
  schema_sql
  schema_columns_sql
  star_sql
  parameter_sql
  sessionparameter_sql
  placeholder_sql
  subquery_sql
  qualify_sql
  set_operations
  union_sql
  union_op
  unnest_sql
  prewhere_sql
  where_sql
  window_sql
  partition_by_sql
  windowspec_sql
  withingroup_sql
  between_sql
  bracket_offset_expressions
  all_sql
  any_sql
  exists_sql
  case_sql
  constraint_sql
  nextvaluefor_sql
  extract_sql
  trim_sql
  convert_concat_args
  concat_sql
  concatws_sql
  check_sql
  foreignkey_sql
  primarykey_sql
  if_sql
  matchagainst_sql
  jsonkeyvalue_sql
  jsonpath_sql
  json_path_part
  formatjson_sql
  jsonobject_sql
  jsonobjectagg_sql
  jsonarray_sql
  jsonarrayagg_sql
  jsoncolumndef_sql
  jsonschema_sql
  jsontable_sql
  openjsoncolumndef_sql
  openjson_sql
  in_sql
  in_unnest_op
  return_sql
  reference_sql
  anonymous_sql
  paren_sql
  neg_sql
  not_sql
  alias_sql
  pivotalias_sql
  aliases_sql
  atindex_sql
  attimezone_sql
  fromtimezone_sql
  add_sql
  and_sql
  or_sql
  xor_sql
  connector_sql
  bitwiseand_sql
  bitwiseleftshift_sql
  bitwisenot_sql
  bitwiseor_sql
  bitwiserightshift_sql
  bitwisexor_sql
  cast_sql
  currentdate_sql
  currenttimestamp_sql
  collate_sql
  command_sql
  comment_sql
  mergetreettlaction_sql
  mergetreettl_sql
  commit_sql
  rollback_sql
  altercolumn_sql
  renametable_sql
  renamecolumn_sql
  altertable_sql
  add_column_sql
  droppartition_sql
  addconstraint_sql
  distinct_sql
  ignorenulls_sql
  respectnulls_sql
  havingmax_sql
  intdiv_sql
  dpipe_sql
  div_sql
  overlaps_sql
  distance_sql
  dot_sql
  eq_sql
  propertyeq_sql
  escape_sql
  glob_sql
  gt_sql
  gte_sql
  ilike_sql
  ilikeany_sql
  is_sql
  like_sql
  likeany_sql
  similarto_sql
  lt_sql
  lte_sql
  mod_sql
  mul_sql
  neq_sql
  nullsafeeq_sql
  nullsafeneq_sql
  slice_sql
  sub_sql
  trycast_sql
  log_sql
  use_sql
  binary
  function_fallback_sql
  func
  format_args
  too_wide
  format_time
  expressions
  op_expressions
  naked_property
  tag_sql
  token_sql
  userdefinedfunction_sql
  joinhint_sql
  kwarg_sql
  when_sql
  merge_sql
  tochar_sql
  tonumber_sql
  dictproperty_sql
  dictrange_sql
  dictsubproperty_sql
  oncluster_sql
  clusteredbyproperty_sql
  anyvalue_sql
  querytransform_sql
  indexconstraintoption_sql
  checkcolumnconstraint_sql
  indexcolumnconstraint_sql
  nvl2_sql
  comprehension_sql
  columnprefix_sql
  opclass_sql
  predict_sql
  forin_sql
  refresh_sql
  operator_sql
  toarray_sql
  tsordstotime_sql
  tsordstotimestamp_sql
  tsordstodate_sql
  unixdate_sql
  lastday_sql
  dateadd_sql
  arrayany_sql
  partitionrange_sql
  truncatetable_sql
  convert_sql